@@ -1,4 +1,4 @@
-/*      $Id: client.h 28362 2008-10-22 19:58:39Z olivier $
+/*      $Id: client.h 27356 2008-07-20 21:22:15Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,15 +12,17 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include <stdlib.h>
@@ -52,7 +54,7 @@
 #ifndef INC_CLIENT_H
 #define INC_CLIENT_H
 
-#define APPLY                           1
+#define APPLY                           +1
 #define REMOVE                          -1
 
 #define PLACEMENT_MOUSE                 0
@@ -83,6 +85,12 @@
                                          UPDATE_MAXIMIZE | \
                                          UPDATE_CACHE)
 
+#define CLIENT_FILL_VERT                (1L<<0)
+#define CLIENT_FILL_HORIZ               (1L<<1)
+#define CLIENT_FILL                     (CLIENT_FILL_VERT | \
+                                         CLIENT_FILL_HORIZ)
+
+
 #define STRUTS_SIZE                     12
 
 #ifndef CLIENT_MIN_VISIBLE
@@ -118,6 +126,8 @@
 #define XFWM_FLAG_FIRST_MAP             (1L<<19)
 #define XFWM_FLAG_LEGACY_FULLSCREEN     (1L<<20)
 #define XFWM_FLAG_MOVING_RESIZING       (1L<<21)
+#define XFWM_FLAG_NEEDS_REDRAW          (1L<<22)
+#define XFWM_FLAG_OPACITY_LOCKED        (1L<<23)
 
 #define CLIENT_FLAG_HAS_STRUT           (1L<<0)
 #define CLIENT_FLAG_HAS_STRUT_PARTIAL   (1L<<1)
@@ -146,7 +156,7 @@
 #define WM_FLAG_CONTEXT_HELP            (1L<<3)
 #define WM_FLAG_URGENT                  (1L<<4)
 
-#define CLIENT_FLAG_INITIAL_VALUES      XFWM_FLAG_HAS_BORDER | \
+#define XFWM_FLAG_INITIAL_VALUES        XFWM_FLAG_HAS_BORDER | \
                                         XFWM_FLAG_HAS_MENU | \
                                         XFWM_FLAG_HAS_MAXIMIZE | \
                                         XFWM_FLAG_HAS_STICK | \
@@ -154,7 +164,8 @@
                                         XFWM_FLAG_HAS_CLOSE | \
                                         XFWM_FLAG_HAS_MOVE | \
                                         XFWM_FLAG_HAS_RESIZE | \
-                                        XFWM_FLAG_FIRST_MAP
+                                        XFWM_FLAG_FIRST_MAP | \
+                                        XFWM_FLAG_NEEDS_REDRAW
 
 #define ALL_WORKSPACES                  (int) 0xFFFFFFFF
 
@@ -195,6 +206,11 @@
                                                                           XFWM_FLAG_IS_RESIZABLE, \
                                                                           XFWM_FLAG_LEGACY_FULLSCREEN) && \
                                          !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
+#define CLIENT_CAN_FILL_WINDOW(c)       (FLAG_TEST_AND_NOT(c->xfwm_flags, XFWM_FLAG_HAS_RESIZE | \
+                                                                          XFWM_FLAG_IS_RESIZABLE, \
+                                                                          XFWM_FLAG_LEGACY_FULLSCREEN) && \
+                                         !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN | CLIENT_FLAG_MAXIMIZED))
+
 #define CLIENT_CAN_STICK_WINDOW(c)      (!(c->transient_for) && \
                                          FLAG_TEST(c->xfwm_flags, XFWM_FLAG_HAS_STICK))
 
@@ -221,14 +237,15 @@ struct _Client
     Window window;
     Window frame;
     Window transient_for;
+    Window user_time_win;
     Window *cmap_windows;
     xfwmWindow title;
-    xfwmWindow sides[3];
-    xfwmWindow corners[4];
+    xfwmWindow sides[SIDE_COUNT];
+    xfwmWindow corners[CORNER_COUNT];
     xfwmWindow buttons[BUTTON_COUNT];
     Window client_leader;
     Window group_leader;
-    xfwmPixmap appmenu[3];
+    xfwmPixmap appmenu[STATE_TOGGLED];
     Colormap cmap;
     unsigned long win_hints;
     unsigned long win_state;
@@ -264,10 +281,10 @@ struct _Client
     int previous_height;
     int initial_layer;
     int ncmap;
-    int button_pressed[BUTTON_COUNT];
+    int button_status[BUTTON_COUNT];
     int struts[STRUTS_SIZE];
     gchar *name;
-    guint32 user_time;
+    Time user_time;
     unsigned long flags;
     unsigned long wm_flags;
     unsigned long xfwm_flags;
@@ -281,7 +298,6 @@ struct _Client
     guint opacity;
     guint opacity_applied;
     guint opacity_flags;
-    gboolean opacity_locked;
 
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
     /* Startup notification */
@@ -305,25 +321,20 @@ Display                 *clientGetXDisplay                      (Client *);
 void                     clientClearLastOpTime                  (Client *);
 void                     clientUpdateWinState                   (Client *,
                                                                  XClientMessageEvent *);
+void                     clientAddUserTimeWin                   (Client *);
+void                     clientRemoveUserTimeWin                (Client *);
 void                     clientUpdateUrgency                    (Client *);
 void                     clientCoordGravitate                   (Client *,
-                                                                 int,
                                                                  int,
                                                                  int *,
                                                                  int *);
 void                     clientAdjustCoordGravity               (Client *,
-                                                                 int,
                                                                  unsigned long *,
                                                                  XWindowChanges *);
-void                     clientGravitate                        (Client *,
-                                                                 int);
 void                     clientConfigure                        (Client *,
                                                                  XWindowChanges *,
                                                                  unsigned long,
                                                                  unsigned short);
-void                     clientMoveResizeWindow                 (Client *,
-                                                                 XWindowChanges *,
-                                                                 unsigned long);
 void                     clientGetMWMHints                      (Client *,
                                                                  gboolean);
 void                     clientGetWMNormalHints                 (Client *,
@@ -344,9 +355,9 @@ void                     clientUpdateAllFrames                  (ScreenInfo *,
                                                                  gboolean);
 void                     clientGrabButtons                      (Client *);
 void                     clientUngrabButtons                    (Client *);
-Client                  *clientGetFromWindow                    (ScreenInfo *,
+Client                  *clientGetFromWindow                    (Client *,
                                                                  Window,
-                                                                 int);
+                                                                 unsigned short);
 void                     clientShow                             (Client *,
                                                                  gboolean);
 void                     clientHide                             (Client *,
@@ -357,7 +368,7 @@ void                     clientHideAll                          (Client *,
 void                     clientClearAllShowDesktop              (ScreenInfo *);
 void                     clientToggleShowDesktop                (ScreenInfo *);
 void                     clientActivate                         (Client *,
-                                                                 guint32);
+                                                                 Time);
 void                     clientClose                            (Client *);
 void                     clientKill                             (Client *);
 void                     clientEnterContextMenuState            (Client *);
@@ -376,14 +387,17 @@ void                     clientUnstick                          (Client *,
 void                     clientToggleSticky                     (Client *,
                                                                  gboolean);
 void                     clientToggleFullscreen                 (Client *);
-void                     clientToggleAbove                      (Client *);
-void                     clientToggleBelow                      (Client *);
+void                     clientToggleLayerAbove                 (Client *);
+void                     clientToggleLayerBelow                 (Client *);
+void                     clientSetLayerNormal                   (Client *);
 void                     clientRemoveMaximizeFlag               (Client *);
 void                     clientToggleMaximized                  (Client *,
                                                                  int,
                                                                  gboolean);
-void                     clientUpdateOpacity                    (ScreenInfo *,
-                                                                 Client *);
+void                     clientFill                             (Client *,
+                                                                 int);
+void                     clientUpdateOpacity                    (Client *);
+void                     clientUpdateAllOpacity                 (ScreenInfo *);
 void                     clientSetOpacity                       (Client *,
                                                                  guint,
                                                                  guint, guint);
@@ -396,16 +410,21 @@ void                     clientResize                           (Client *,
                                                                  int,
                                                                  XEvent *);
 void                     clientCycle                            (Client *,
-                                                                 XEvent *);
+                                                                 XKeyEvent *);
 void                     clientButtonPress                      (Client *,
                                                                  Window,
                                                                  XButtonEvent *);
+xfwmPixmap *             clientGetButtonPixmap                  (Client *,
+                                                                 int,
+                                                                 int);
+int                      clientGetButtonState                   (Client *,
+                                                                 int,
+                                                                 int);
 Client                  *clientGetLeader                        (Client *);
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
 char                    *clientGetStartupId                     (Client *);
 #endif /* HAVE_LIBSTARTUP_NOTIFICATION */
 #ifdef HAVE_XSYNC
-void                     clientXSyncClearTimeout                (Client *);
 void                     clientXSyncRequest                     (Client *);
 #endif /* HAVE_XSYNC */
 
