@@ -1,4 +1,4 @@
-/*      $Id: compositor.c 27454 2008-08-15 21:45:57Z olivier $
+/*      $Id: compositor.c 27803 2008-09-10 16:07:00Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,7 +12,9 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         xcompmgr - (c) 2003 Keith Packard
         metacity - (c) 2003, 2004 Red Hat, Inc.
@@ -69,11 +71,11 @@
                                          !FLAG_TEST (cw->c->flags, CLIENT_FLAG_FULLSCREEN))
 #define WIN_NO_SHADOW(cw)               ((cw->c) && \
                                            (FLAG_TEST (cw->c->flags, CLIENT_FLAG_FULLSCREEN | CLIENT_FLAG_BELOW) || \
-                                              (cw->c->type & (WINDOW_DOCK | WINDOW_DESKTOP))))
+                                            (cw->c->type & WINDOW_DESKTOP)))
 #define WIN_IS_DOCK(cw)                 (WIN_HAS_CLIENT(cw) && (cw->c->type & WINDOW_DOCK))
 #define WIN_IS_OVERRIDE(cw)             (cw->attr.override_redirect)
 #define WIN_IS_ARGB(cw)                 (cw->argb)
-#define WIN_IS_OPAQUE(cw)               (((cw->opacity == NET_WM_OPAQUE) && !WIN_IS_ARGB(cw)) || (cw->screen_info->wins_unredirected))
+#define WIN_IS_OPAQUE(cw)               ((cw->opacity == NET_WM_OPAQUE) && !WIN_IS_ARGB(cw))
 #define WIN_IS_NATIVE_OPAQUE(cw)        ((cw->native_opacity) && !WIN_IS_ARGB(cw))
 #define WIN_IS_FULLSCREEN(cw)           ((cw->attr.x <= 0) && \
                                            (cw->attr.y <= 0) && \
@@ -87,7 +89,8 @@
 #define WIN_IS_DAMAGED(cw)              (cw->damaged)
 #define WIN_IS_REDIRECTED(cw)           (cw->redirected)
 
-#define USE_IDLE_REPAINT
+/* Set TIMEOUT_REPAINT to 0 to disable timeout repaint */
+#define TIMEOUT_REPAINT       20 /* (1/50 sec.) */
 
 typedef struct _CWindow CWindow;
 struct _CWindow
@@ -935,7 +938,12 @@ win_extents (CWindow *cw)
               !WIN_IS_OVERRIDE(cw) &&
               !WIN_NO_SHADOW(cw) &&
               !WIN_IS_DOCK(cw) &&
-              (WIN_HAS_FRAME(cw) || !(WIN_IS_ARGB(cw) || WIN_IS_SHAPED(cw)))))
+              (WIN_HAS_FRAME(cw) || !(WIN_IS_ARGB(cw) || WIN_IS_SHAPED(cw)))) ||
+          (screen_info->params->show_dock_shadow &&
+              WIN_IS_DOCK(cw) &&
+              !WIN_NO_SHADOW(cw) &&
+              !WIN_IS_OVERRIDE(cw) &&
+              (!(WIN_IS_ARGB(cw) || WIN_IS_SHAPED(cw)))))
     {
         XRectangle sr;
 
@@ -1083,8 +1091,6 @@ unredirect_win (CWindow *cw)
         cw->redirected = FALSE;
 
         XCompositeUnredirectWindow (display_info->dpy, cw->id, display_info->composite_mode);
-        XSync (display_info->dpy, FALSE);
-
         TRACE ("Window 0x%lx unredirected, wins_unredirected is %i", cw->id, screen_info->wins_unredirected);
     }
 }
@@ -1385,6 +1391,18 @@ paint_all (ScreenInfo *screen_info, XserverRegion region)
     XFixesDestroyRegion (dpy, paint_region);
 }
 
+#if TIMEOUT_REPAINT
+static void
+remove_timeouts (ScreenInfo *screen_info)
+{
+    if (screen_info->compositor_timeout_id != 0)
+    {
+        g_source_remove (screen_info->compositor_timeout_id);
+        screen_info->compositor_timeout_id = 0;
+    }
+}
+#endif /* TIMEOUT_REPAINT */
+
 static void
 repair_screen (ScreenInfo *screen_info)
 {
@@ -1398,6 +1416,10 @@ repair_screen (ScreenInfo *screen_info)
         return;
     }
 
+#if TIMEOUT_REPAINT
+    remove_timeouts (screen_info);
+#endif /* TIMEOUT_REPAINT */
+
     display_info = screen_info->display_info;
     if (screen_info->allDamage != None)
     {
@@ -1407,24 +1429,35 @@ repair_screen (ScreenInfo *screen_info)
     }
 }
 
-#ifdef USE_IDLE_REPAINT
-static void
-remove_timeouts (DisplayInfo *display_info)
+#if TIMEOUT_REPAINT
+static gboolean
+compositor_timeout_cb (gpointer data)
 {
-    if (display_info->compositor_idle_id != 0)
-    {
-        g_source_remove (display_info->compositor_idle_id);
-        display_info->compositor_idle_id = 0;
-    }
+    ScreenInfo *screen_info;
 
-    if (display_info->compositor_timeout_id != 0)
-    {
-        g_source_remove (display_info->compositor_timeout_id);
-        display_info->compositor_timeout_id = 0;
-    }
+    screen_info = (ScreenInfo *) data;
+    screen_info->compositor_timeout_id = 0;
+    repair_screen (screen_info);
+
+    return FALSE;
 }
-#endif /* USE_IDLE_REPAINT */
+#endif /* TIMEOUT_REPAINT */
 
+static void
+add_repair (ScreenInfo *screen_info)
+{
+#if TIMEOUT_REPAINT
+    if (screen_info->compositor_timeout_id != 0)
+    {
+        return;
+    }
+    screen_info->compositor_timeout_id =
+        g_timeout_add (TIMEOUT_REPAINT,
+                       compositor_timeout_cb, screen_info);
+#endif /* TIMEOUT_REPAINT */
+}
+
+#if TIMEOUT_REPAINT == 0
 static void
 repair_display (DisplayInfo *display_info)
 {
@@ -1433,61 +1466,12 @@ repair_display (DisplayInfo *display_info)
     g_return_if_fail (display_info);
     TRACE ("entering repair_display");
 
-#ifdef USE_IDLE_REPAINT
-    remove_timeouts (display_info);
-#endif /* USE_IDLE_REPAINT */
     for (screens = display_info->screens; screens; screens = g_slist_next (screens))
     {
-        repair_screen ((ScreenInfo *) screens->data);
+        add_repair ((ScreenInfo *) screens->data);
     }
 }
-
-#ifdef USE_IDLE_REPAINT
-static gboolean
-compositor_idle_cb (gpointer data)
-{
-    DisplayInfo *display_info = (DisplayInfo *) data;
-
-    display_info->compositor_idle_id = 0;
-    repair_display (display_info);
-
-    return FALSE;
-}
-
-static gboolean
-compositor_timeout_cb (gpointer data)
-{
-    DisplayInfo *display_info;
-
-    display_info = (DisplayInfo *) data;
-    display_info->compositor_timeout_id = 0;
-    repair_display (display_info);
-
-    return FALSE;
-}
-#endif /* USE_IDLE_REPAINT */
-
-static void
-add_repair (DisplayInfo *display_info)
-{
-#ifdef USE_IDLE_REPAINT
-    if (display_info->compositor_idle_id != 0)
-    {
-        return;
-    }
-    display_info->compositor_idle_id =
-        g_idle_add_full (G_PRIORITY_HIGH_IDLE,
-                         compositor_idle_cb, display_info, NULL);
-    if (display_info->compositor_timeout_id != 0)
-    {
-        return;
-    }
-
-    display_info->compositor_timeout_id =
-        g_timeout_add (50 /* ms */,
-                       compositor_timeout_cb, display_info);
-#endif /* USE_IDLE_REPAINT */
-}
+#endif
 
 static void
 add_damage (ScreenInfo *screen_info, XserverRegion damage)
@@ -1516,7 +1500,7 @@ add_damage (ScreenInfo *screen_info, XserverRegion damage)
     }
 
     /* The per-screen allDamage region is freed by repair_screen () */
-    add_repair (screen_info->display_info);
+    add_repair (screen_info);
 }
 
 static void
@@ -1747,7 +1731,7 @@ set_win_opacity (CWindow *cw, guint opacity)
             XFixesDestroyRegion (display_info->dpy, cw->extents);
         }
         cw->extents = win_extents (cw);
-        add_repair (display_info);
+        add_repair (screen_info);
     }
 }
 
@@ -1846,7 +1830,6 @@ unmap_win (CWindow *cw)
             }
 #endif /* HAVE_OVERLAYS */
             damage_screen (screen_info);
-            repair_screen (screen_info);
        }
     }
     else if (WIN_IS_VISIBLE(cw))
@@ -1879,7 +1862,7 @@ init_opacity (CWindow *cw)
     cw->native_opacity = FALSE;
     if (c)
     {
-        cw->opacity_locked = c->opacity_locked;
+        cw->opacity_locked = FLAG_TEST (c->xfwm_flags, XFWM_FLAG_OPACITY_LOCKED);
         cw->opacity = c->opacity_applied;
         cw->native_opacity = WIN_IS_OPAQUE(cw);
     }
@@ -2059,7 +2042,7 @@ restack_win (CWindow *cw, Window above)
 }
 
 static void
-resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw, gboolean shape_notify)
+resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw)
 {
     DisplayInfo *display_info;
     ScreenInfo *screen_info;
@@ -2088,7 +2071,7 @@ resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw, gbool
         cw->extents = None;
     }
 
-    if ((cw->attr.width != width) || (cw->attr.height != height) || shape_notify)
+    if ((cw->attr.width != width) || (cw->attr.height != height))
     {
 #if HAVE_NAME_WINDOW_PIXMAP
         if (cw->name_window_pixmap)
@@ -2111,7 +2094,7 @@ resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw, gbool
     }
 
     if ((cw->attr.width != width) || (cw->attr.height != height) ||
-        (cw->attr.x != x) || (cw->attr.y != y) || shape_notify)
+        (cw->attr.x != x) || (cw->attr.y != y))
     {
         if (cw->borderSize)
         {
@@ -2136,12 +2119,70 @@ resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw, gbool
     {
         cw->extents = win_extents (cw);
         XFixesUnionRegion (display_info->dpy, damage, damage, cw->extents);
-        /* A shape notify will likely change the shadows too, so clear the extents */
-        if (shape_notify)
+
+        fix_region (cw, damage);
+        /* damage region will be destroyed by add_damage () */
+        add_damage (screen_info, damage);
+    }
+}
+
+static void
+reshape_win (CWindow *cw)
+{
+    DisplayInfo *display_info;
+    ScreenInfo *screen_info;
+    XserverRegion damage;
+
+    g_return_if_fail (cw != NULL);
+    TRACE ("entering reshape_win");
+
+    screen_info = cw->screen_info;
+    display_info = screen_info->display_info;
+
+    damage = None;
+
+    if (WIN_IS_VISIBLE(cw))
+    {
+        damage = XFixesCreateRegion (display_info->dpy, NULL, 0);
+        if (cw->extents)
         {
-            XFixesDestroyRegion (display_info->dpy, cw->extents);
-            cw->extents = None;
+            XFixesCopyRegion (display_info->dpy, damage, cw->extents);
         }
+    }
+
+    if (cw->extents)
+    {
+        XFixesDestroyRegion (display_info->dpy, cw->extents);
+        cw->extents = None;
+    }
+
+    if (cw->shadow)
+    {
+        XRenderFreePicture (display_info->dpy, cw->shadow);
+        cw->shadow = None;
+    }
+
+    if (cw->borderSize)
+    {
+        XFixesDestroyRegion (display_info->dpy, cw->borderSize);
+        cw->borderSize = None;
+    }
+
+    if (cw->clientSize)
+    {
+        XFixesDestroyRegion (display_info->dpy, cw->clientSize);
+        cw->clientSize = None;
+    }
+
+    if (damage)
+    {
+        cw->extents = win_extents (cw);
+        XFixesUnionRegion (display_info->dpy, damage, damage, cw->extents);
+
+        /* A shape notify will likely change the shadows too, so clear the extents */
+        XFixesDestroyRegion (display_info->dpy, cw->extents);
+        cw->extents = None;
+
         fix_region (cw, damage);
         /* damage region will be destroyed by add_damage () */
         add_damage (screen_info, damage);
@@ -2176,6 +2217,7 @@ destroy_win (DisplayInfo *display_info, Window id)
 static void
 compositorHandleDamage (DisplayInfo *display_info, XDamageNotifyEvent *ev)
 {
+    ScreenInfo *screen_info;
     CWindow *cw;
 
     g_return_if_fail (display_info != NULL);
@@ -2192,15 +2234,9 @@ compositorHandleDamage (DisplayInfo *display_info, XDamageNotifyEvent *ev)
     cw = find_cwindow_in_display (display_info, ev->drawable);
     if ((cw) && WIN_IS_REDIRECTED(cw))
     {
+        screen_info = cw->screen_info;
         repair_win (cw, &ev->area);
-        display_info->damages_pending = ev->more;
-#ifdef USE_IDLE_REPAINT
-        /* If there are more damages to come, we'll schedule the repair later */
-        if (!display_info->damages_pending)
-        {
-            add_repair (display_info);
-        }
-#endif /* USE_IDLE_REPAINT */
+        screen_info->damages_pending = ev->more;
     }
 }
 
@@ -2230,7 +2266,7 @@ compositorHandlePropertyNotify (DisplayInfo *display_info, XPropertyEvent *ev)
                 XClearArea (display_info->dpy, screen_info->output, 0, 0, 0, 0, TRUE);
                 XRenderFreePicture (display_info->dpy, screen_info->rootTile);
                 screen_info->rootTile = None;
-                add_repair (display_info);
+                add_repair (screen_info);
 
                 return;
             }
@@ -2280,7 +2316,14 @@ compositorHandlePropertyNotify (DisplayInfo *display_info, XPropertyEvent *ev)
             cw->opacity_locked = getOpacityLock (display_info, cw->id);
             if (cw->c)
             {
-                cw->c->opacity_locked = cw->opacity_locked;
+                if (cw->opacity_locked)
+                {
+                    FLAG_SET (cw->c->xfwm_flags, XFWM_FLAG_OPACITY_LOCKED);
+                }
+                else
+                {
+                    FLAG_UNSET (cw->c->xfwm_flags, XFWM_FLAG_OPACITY_LOCKED);
+                }
             }
         }
     }
@@ -2337,7 +2380,7 @@ compositorHandleConfigureNotify (DisplayInfo *display_info, XConfigureEvent *ev)
     if (cw)
     {
         restack_win (cw, ev->above);
-        resize_win (cw, ev->x, ev->y, ev->width, ev->height, ev->border_width, FALSE);
+        resize_win (cw, ev->x, ev->y, ev->width, ev->height, ev->border_width);
     }
 }
 
@@ -2371,7 +2414,6 @@ compositorHandleCirculateNotify (DisplayInfo *display_info, XCirculateEvent *ev)
         above = None;
     }
     restack_win (cw, above);
-    add_repair (display_info);
 }
 
 static void
@@ -2481,9 +2523,7 @@ compositorHandleShapeNotify (DisplayInfo *display_info, XShapeEvent *ev)
             {
                 cw->shaped = FALSE;
             }
-            resize_win (cw, cw->attr.x, cw->attr.y,
-                            ev->width + ev->x, ev->height + ev->y,
-                            cw->attr.border_width, TRUE);
+            reshape_win  (cw);
             if ((ev->shaped) && !(cw->shaped))
             {
                 cw->shaped = TRUE;
@@ -2644,7 +2684,7 @@ compositorResizeWindow (DisplayInfo *display_info, Window id, int x, int y, int
     cw = find_cwindow_in_display (display_info, id);
     if (cw)
     {
-        resize_win (cw, x, y, width, height, 0, FALSE);
+        resize_win (cw, x, y, width, height, 0);
     }
 #endif /* HAVE_COMPOSITOR */
 }
@@ -2705,12 +2745,9 @@ compositorHandleEvent (DisplayInfo *display_info, XEvent *ev)
     {
         compositorHandleShapeNotify (display_info, (XShapeEvent *) ev);
     }
-#ifndef USE_IDLE_REPAINT
-    if (!display_info->damages_pending)
-    {
-        repair_display (display_info);
-    }
-#endif /* USE_IDLE_REPAINT */
+#if TIMEOUT_REPAINT == 0
+    repair_display (display_info);
+#endif /* TIMEOUT_REPAINT */
 
 #endif /* HAVE_COMPOSITOR */
 }
@@ -2780,10 +2817,6 @@ compositorInitDisplay (DisplayInfo *display_info)
 #endif /* DEBUG */
     }
 
-    display_info->compositor_idle_id = 0;
-    display_info->compositor_timeout_id = 0;
-    display_info->damages_pending = FALSE;
-
     display_info->enable_compositor = ((display_info->have_render)
                                     && (display_info->have_composite)
                                     && (display_info->have_damage)
@@ -2923,6 +2956,8 @@ compositorManageScreen (ScreenInfo *screen_info)
     screen_info->cwindows = NULL;
     screen_info->compositor_active = TRUE;
     screen_info->wins_unredirected = 0;
+    screen_info->compositor_timeout_id = 0;
+    screen_info->damages_pending = FALSE;
 
     XClearArea (display_info->dpy, screen_info->output, 0, 0, 0, 0, TRUE);
     compositorSetCMSelection (screen_info, screen_info->xfwm4_win);
@@ -2958,6 +2993,10 @@ compositorUnmanageScreen (ScreenInfo *screen_info)
     }
     screen_info->compositor_active = FALSE;
 
+#if TIMEOUT_REPAINT
+    remove_timeouts (screen_info);
+#endif /* TIMEOUT_REPAINT */
+
     i = 0;
     for (index = screen_info->cwindows; index; index = g_list_next (index))
     {
@@ -3035,7 +3074,6 @@ compositorAddAllWindows (ScreenInfo *screen_info)
         return;
     }
 
-    XSync (display_info->dpy, FALSE);
     myDisplayGrabServer (display_info);
     XQueryTree (display_info->dpy, screen_info->xroot, &w1, &w2, &wins, &count);
 
@@ -3043,7 +3081,7 @@ compositorAddAllWindows (ScreenInfo *screen_info)
     {
         Client *c;
 
-        c = clientGetFromWindow (screen_info, wins[i], FRAME);
+        c = myScreenGetClientFromWindow (screen_info, wins[i], SEARCH_FRAME);
         compositorAddWindow (display_info, wins[i], c);
     }
     if (wins)
@@ -3051,7 +3089,6 @@ compositorAddAllWindows (ScreenInfo *screen_info)
         XFree (wins);
     }
     myDisplayUngrabServer (display_info);
-    XSync (display_info->dpy, FALSE);
 #endif /* HAVE_COMPOSITOR */
 }
 
