@@ -1,4 +1,4 @@
-/*      $Id: display.c 28362 2008-10-22 19:58:39Z olivier $
+/*      $Id: display.c 27356 2008-07-20 21:22:15Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,14 +12,16 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include <X11/X.h>
@@ -61,7 +63,7 @@ handleXError (Display * dpy, XErrorEvent * err)
     char buf[64];
 
     XGetErrorText (dpy, err->error_code, buf, 63);
-    g_print ("XError: %s\n", buf);                                    
+    g_print ("XError: %s\n", buf);
     g_print ("==>  XID 0x%lx, Request %d, Error %d <==\n",
               err->resourceid, err->request_code, err->error_code);
 #endif
@@ -88,7 +90,6 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_NET_DESKTOP_NAMES",
         "_NET_DESKTOP_VIEWPORT",
         "_NET_FRAME_EXTENTS",
-        "_NET_MOVERESIZE_WINDOW",
         "_NET_NUMBER_OF_DESKTOPS",
         "_NET_REQUEST_FRAME_EXTENTS",
         "_NET_SHOWING_DESKTOP",
@@ -96,10 +97,14 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_NET_SUPPORTED",
         "_NET_SUPPORTING_WM_CHECK",
         "_NET_SYSTEM_TRAY_OPCODE",
+        "_NET_WM_ACTION_ABOVE",
+        "_NET_WM_ACTION_BELOW",
         "_NET_WM_ACTION_CHANGE_DESKTOP",
         "_NET_WM_ACTION_CLOSE",
+        "_NET_WM_ACTION_FULLSCREEN",
         "_NET_WM_ACTION_MAXIMIZE_HORZ",
         "_NET_WM_ACTION_MAXIMIZE_VERT",
+        "_NET_WM_ACTION_MINIMIZE",
         "_NET_WM_ACTION_MOVE",
         "_NET_WM_ACTION_RESIZE",
         "_NET_WM_ACTION_SHADE",
@@ -132,6 +137,7 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_NET_WM_SYNC_REQUEST",
         "_NET_WM_SYNC_REQUEST_COUNTER",
         "_NET_WM_USER_TIME",
+        "_NET_WM_USER_TIME_WINDOW",
         "_NET_WM_WINDOW_TYPE",
         "_NET_WM_WINDOW_TYPE_DESKTOP",
         "_NET_WM_WINDOW_TYPE_DIALOG",
@@ -186,7 +192,7 @@ myDisplayCreateTimestampWin (DisplayInfo *display_info)
 
     attributes.event_mask = PropertyChangeMask;
     attributes.override_redirect = TRUE;
-    display_info->timestamp_win = 
+    display_info->timestamp_win =
         XCreateWindow (display_info->dpy, DefaultRootWindow (display_info->dpy),
                        -100, -100, 10, 10, 0, 0, CopyFromParent, CopyFromParent,
                        CWEventMask | CWOverrideRedirect, &attributes);
@@ -239,18 +245,18 @@ myDisplayInit (GdkDisplay *gdisplay)
 
 #ifdef HAVE_XSYNC
     display->have_xsync = FALSE;
-    
+
     display->xsync_error_base = 0;
     display->xsync_event_base = 0;
 
     major = SYNC_MAJOR_VERSION;
     minor = SYNC_MINOR_VERSION;
-    
+
     if (XSyncQueryExtension (display->dpy,
                               &display->xsync_event_base,
                               &display->xsync_error_base)
-         && XSyncInitialize (display->dpy, 
-                             &major, 
+         && XSyncInitialize (display->dpy,
+                             &major,
                              &minor))
     {
         display->have_xsync = TRUE;
@@ -365,7 +371,7 @@ myDisplayClose (DisplayInfo *display)
         display->hostname = NULL;
     }
 
-    for (i = 0; i < 7; i++)
+    for (i = 0; i < SIDE_COUNT + CORNER_COUNT; i++)
     {
         XFreeCursor (display->dpy, display->resize_cursor[i]);
         display->resize_cursor[i] = None;
@@ -484,7 +490,7 @@ myDisplayRemoveClient (DisplayInfo *display, Client *c)
 }
 
 Client *
-myDisplayGetClientFromWindow (DisplayInfo *display, Window w, int mode)
+myDisplayGetClientFromWindow (DisplayInfo *display, Window w, unsigned short mode)
 {
     GSList *index;
 
@@ -494,30 +500,9 @@ myDisplayGetClientFromWindow (DisplayInfo *display, Window w, int mode)
     for (index = display->clients; index; index = g_slist_next (index))
     {
         Client *c = (Client *) index->data;
-        switch (mode)
+        if (clientGetFromWindow (c, w, mode))
         {
-            case WINDOW:
-                if (c->window == w)
-                {
-                    TRACE ("found \"%s\" (mode WINDOW)", c->name);
-                    return (c);
-                }
-                break;
-            case FRAME:
-                if (c->frame == w)
-                {
-                    TRACE ("found \"%s\" (mode FRAME)", c->name);
-                    return (c);
-                }
-                break;
-            case ANY:
-            default:
-                if ((c->frame == w) || (c->window == w))
-                {
-                    TRACE ("found \"%s\" (mode ANY)", c->name);
-                    return (c);
-                }
-                break;
+            return (c);
         }
     }
     TRACE ("no client found");
@@ -655,7 +640,7 @@ myDisplayGetClientFromXSyncAlarm (DisplayInfo *display, XSyncAlarm alarm)
 
     g_return_val_if_fail (alarm != None, NULL);
     g_return_val_if_fail (display != NULL, NULL);
-    
+
     for (index = display->clients; index; index = g_slist_next (index))
     {
         Client *c = (Client *) index->data;
@@ -686,84 +671,98 @@ myDisplayGetDefaultScreen (DisplayInfo *display)
     return NULL;
 }
 
-guint32
+Time
 myDisplayUpdateCurrentTime (DisplayInfo *display, XEvent *ev)
 {
-    g_return_val_if_fail (display != NULL, CurrentTime);
+    g_return_val_if_fail (display != NULL, (Time) CurrentTime);
 
     switch (ev->type)
     {
         case KeyPress:
         case KeyRelease:
-            display->current_time = (guint32) ev->xkey.time;
+            display->current_time = (Time) ev->xkey.time;
             break;
         case ButtonPress:
         case ButtonRelease:
-            display->current_time = (guint32) ev->xbutton.time;
+            display->current_time = (Time) ev->xbutton.time;
             break;
         case MotionNotify:
-            display->current_time = (guint32) ev->xmotion.time;
+            display->current_time = (Time) ev->xmotion.time;
             break;
         case EnterNotify:
         case LeaveNotify:
-            display->current_time = (guint32) ev->xcrossing.time;
+            display->current_time = (Time) ev->xcrossing.time;
             break;
         case PropertyNotify:
-            display->current_time = (guint32) ev->xproperty.time;
+            display->current_time = (Time) ev->xproperty.time;
             break;
         case SelectionClear:
-            display->current_time = (guint32) ev->xselectionclear.time;
+            display->current_time = (Time) ev->xselectionclear.time;
             break;
         case SelectionRequest:
-            display->current_time = (guint32) ev->xselectionrequest.time;
+            display->current_time = (Time) ev->xselectionrequest.time;
             break;
         case SelectionNotify:
-            display->current_time = (guint32) ev->xselection.time;
+            display->current_time = (Time) ev->xselection.time;
             break;
         default:
-            display->current_time = (guint32) CurrentTime;
+            display->current_time = (Time) CurrentTime;
             break;
     }
     return display->current_time;
 }
 
-guint32
+Time
 myDisplayGetCurrentTime (DisplayInfo *display)
 {
-    g_return_val_if_fail (display != NULL, CurrentTime);
+    g_return_val_if_fail (display != NULL, (Time) CurrentTime);
 
-    TRACE ("myDisplayGetCurrentTime gives timestamp=%u", (guint32) display->current_time);
-    return display->current_time;
+    TRACE ("myDisplayGetCurrentTime gives timestamp=%u", (unsigned int) display->current_time);
+    return (Time) display->current_time;
 }
 
-guint32
-myDisplayGetTime (DisplayInfo * display, guint32 timestamp)
+Time
+myDisplayGetTime (DisplayInfo * display, Time timestamp)
 {
-    guint32 time;
+    Time time;
 
     time = timestamp;
-    if (time == CurrentTime)
+    if (time == (Time) CurrentTime)
     {
         time = getXServerTime (display);
     }
 
-    TRACE ("myDisplayGetTime gives timestamp=%u", (guint32) time);
-    return time;
+    TRACE ("myDisplayGetTime gives timestamp=%u", (unsigned int) time);
+    return (time);
 }
 
-guint32
+Time
 myDisplayGetLastUserTime (DisplayInfo *display)
 {
-    g_return_val_if_fail (display != NULL, CurrentTime);
+    g_return_val_if_fail (display != NULL, (Time) CurrentTime);
 
-    TRACE ("myDisplayGetLastUserTime gives timestamp=%u", (guint32) display->last_user_time);
-    return display->last_user_time;
+    TRACE ("myDisplayGetLastUserTime gives timestamp=%u", (unsigned int) display->last_user_time);
+    return (Time) display->last_user_time;
 }
 
 void
-myDisplaySetLastUserTime (DisplayInfo *display, guint32 timestamp)
+myDisplaySetLastUserTime (DisplayInfo *display, Time timestamp)
 {
     g_return_if_fail (display != NULL);
+    g_return_if_fail (timestamp != (Time) 0);
+
+    if (TIMESTAMP_IS_BEFORE(timestamp, display->last_user_time))
+    {
+        g_warning ("Last user time set back to %u (was %u)", (unsigned int) timestamp, (unsigned int) display->last_user_time);
+    }
+    display->last_user_time = timestamp;
+}
+
+void
+myDisplayUpdateLastUserTime (DisplayInfo *display, Time timestamp)
+{
+    g_return_if_fail (display != NULL);
+    g_return_if_fail (timestamp != (Time) 0);
 
     if (TIMESTAMP_IS_BEFORE(display->last_user_time, timestamp))
     {
