@@ -1,4 +1,4 @@
-/*      $Id: display.h 28362 2008-10-22 19:58:39Z olivier $
+/*      $Id: display.h 27434 2008-08-09 15:43:06Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,9 +12,11 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
@@ -22,7 +24,7 @@
 #define INC_DISPLAY_H
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include <X11/X.h>
@@ -31,6 +33,10 @@
 #include <X11/cursorfont.h>
 #include <X11/extensions/shape.h>
 
+#ifndef ShapeInput
+#define ShapeInput 2
+#endif
+
 #ifdef HAVE_RANDR
 #include <X11/extensions/Xrandr.h>
 #endif /* HAVE_RANDR */
@@ -66,18 +72,36 @@
  * if that needs some explanation, please refer to metacity's
  * display.h source where it is explaned
  */
-#define TIMESTAMP_IS_BEFORE_REAL(time1, time2)  ((((time1) < (time2)) && ((time2) - (time1) < ((guint32)-1)/2 )) || \
-                                                 (((time1) > (time2)) && ((time1) - (time2) > ((guint32)-1)/2 )))
-#define TIMESTAMP_IS_BEFORE(time1, time2)       ((time1) == 0 ||                                                    \
+#define TIMESTAMP_IS_BEFORE_REAL(time1, time2)  (((time1 < time2) && (time2 - time1 < (G_MAXUINT32 >> 1))) || \
+                                                 ((time1 > time2) && (time1 - time2 > (G_MAXUINT32 >> 1))))
+#define TIMESTAMP_IS_BEFORE(time1, time2)       ((time1 == 0) ||                                                    \
                                                 (TIMESTAMP_IS_BEFORE_REAL(time1, time2) &&                          \
-                                                (time2) != 0))
+                                                (time2 != 0)))
 
 enum
 {
-    CORNER_TOP_LEFT = 0,
-    CORNER_TOP_RIGHT,
-    CORNER_BOTTOM_LEFT,
+    SEARCH_WINDOW         = (1 << 0),
+    SEARCH_FRAME          = (1 << 1),
+    SEARCH_BUTTON         = (1 << 2),
+    SEARCH_WIN_USER_TIME  = (1 << 3)
+};
+
+enum
+{
+    TITLE_1 = 0,
+    TITLE_2,
+    TITLE_3,
+    TITLE_4,
+    TITLE_5,
+    TITLE_COUNT
+};
+
+enum
+{
+    CORNER_BOTTOM_LEFT = 0,
     CORNER_BOTTOM_RIGHT,
+    CORNER_TOP_LEFT,
+    CORNER_TOP_RIGHT,
     CORNER_COUNT
 };
 
@@ -89,12 +113,42 @@ enum
     SIDE_TOP,
     SIDE_COUNT
 };
+#define NO_HANDLE -1
+#define HANDLES_COUNT (CORNER_COUNT + SIDE_COUNT - 1)
 
 enum
 {
-   ANY,
-   WINDOW,
-   FRAME
+    MENU_BUTTON = 0,
+    STICK_BUTTON,
+    SHADE_BUTTON,
+    HIDE_BUTTON,
+    MAXIMIZE_BUTTON,
+    CLOSE_BUTTON,
+    TITLE_SEPARATOR,
+    BUTTON_STRING_COUNT
+};
+#define BUTTON_COUNT (BUTTON_STRING_COUNT - 1)
+
+enum
+{
+    ACTIVE = 0,
+    INACTIVE,
+    PRELIGHT,
+    PRESSED,
+    T_ACTIVE,
+    T_INACTIVE,
+    T_PRELIGHT,
+    T_PRESSED,
+    STATE_COUNT
+};
+#define STATE_TOGGLED (STATE_COUNT / 2)
+
+enum
+{
+    BUTTON_STATE_NORMAL = 0,
+    BUTTON_STATE_PRELIGHT,
+    BUTTON_STATE_PRESSED,
+    BUTTON_STATE_COUNT
 };
 
 enum
@@ -130,7 +184,6 @@ enum
     NET_DESKTOP_NAMES,
     NET_DESKTOP_VIEWPORT,
     NET_FRAME_EXTENTS,
-    NET_MOVERESIZE_WINDOW,
     NET_NUMBER_OF_DESKTOPS,
     NET_REQUEST_FRAME_EXTENTS,
     NET_SHOWING_DESKTOP,
@@ -138,10 +191,14 @@ enum
     NET_SUPPORTED,
     NET_SUPPORTING_WM_CHECK,
     NET_SYSTEM_TRAY_OPCODE,
+    NET_WM_ACTION_ABOVE,
+    NET_WM_ACTION_BELOW,
     NET_WM_ACTION_CHANGE_DESKTOP,
     NET_WM_ACTION_CLOSE,
+    NET_WM_ACTION_FULLSCREEN,
     NET_WM_ACTION_MAXIMIZE_HORZ,
     NET_WM_ACTION_MAXIMIZE_VERT,
+    NET_WM_ACTION_MINIMIZE,
     NET_WM_ACTION_MOVE,
     NET_WM_ACTION_RESIZE,
     NET_WM_ACTION_SHADE,
@@ -174,6 +231,7 @@ enum
     NET_WM_SYNC_REQUEST,
     NET_WM_SYNC_REQUEST_COUNTER,
     NET_WM_USER_TIME,
+    NET_WM_USER_TIME_WINDOW,
     NET_WM_WINDOW_TYPE,
     NET_WM_WINDOW_TYPE_DESKTOP,
     NET_WM_WINDOW_TYPE_DIALOG,
@@ -236,7 +294,7 @@ struct _DisplayInfo
     Cursor busy_cursor;
     Cursor move_cursor;
     Cursor root_cursor;
-    Cursor resize_cursor[8];
+    Cursor resize_cursor[SIDE_COUNT + CORNER_COUNT];
 
     Atom atoms[ATOM_COUNT];
 
@@ -255,8 +313,8 @@ struct _DisplayInfo
     gint nb_screens;
     gchar* hostname;
 
-    guint32 current_time;
-    guint32 last_user_time;
+    Time current_time;
+    Time last_user_time;
 
     gboolean enable_compositor;
 #ifdef HAVE_RENDER
@@ -283,10 +341,6 @@ struct _DisplayInfo
     gboolean have_composite;
     gboolean have_damage;
     gboolean have_fixes;
-    gboolean damages_pending;
-
-    guint compositor_idle_id;
-    guint compositor_timeout_id;
 
 #if HAVE_NAME_WINDOW_PIXMAP
     gboolean have_name_window_pixmap;
@@ -316,7 +370,7 @@ void                     myDisplayRemoveClient                  (DisplayInfo *,
                                                                  Client *);
 Client                  *myDisplayGetClientFromWindow           (DisplayInfo *,
                                                                  Window,
-                                                                 int);
+                                                                 unsigned short);
 void                     myDisplayAddScreen                     (DisplayInfo *,
                                                                  ScreenInfo *);
 void                     myDisplayRemoveScreen                  (DisplayInfo *,
@@ -338,14 +392,16 @@ Client                  *myDisplayGetClientFromXSyncAlarm       (DisplayInfo *,
                                                                  XSyncAlarm);
 #endif /* HAVE_XSYNC */
 ScreenInfo              *myDisplayGetDefaultScreen              (DisplayInfo *);
-guint32                  myDisplayUpdateCurrentTime             (DisplayInfo *,
+Time                     myDisplayUpdateCurrentTime             (DisplayInfo *,
                                                                  XEvent *);
-guint32                  myDisplayGetCurrentTime                (DisplayInfo *);
-guint32                  myDisplayGetTime                       (DisplayInfo *,
-                                                                 guint32);
-guint32                  myDisplayGetLastUserTime               (DisplayInfo *);
+Time                     myDisplayGetCurrentTime                (DisplayInfo *);
+Time                     myDisplayGetTime                       (DisplayInfo *,
+                                                                 Time);
+Time                     myDisplayGetLastUserTime               (DisplayInfo *);
 void                     myDisplaySetLastUserTime               (DisplayInfo *,
-                                                                 guint32);
+                                                                 Time);
+void                     myDisplayUpdateLastUserTime            (DisplayInfo *,
+                                                                 Time);
 gboolean                 myDisplayTestXrender                   (DisplayInfo *,
                                                                  gdouble);
 
