@@ -1,4 +1,4 @@
-/*      $Id: events.h 24099 2006-12-13 22:01:17Z olivier $
+/*      $Id: events.h 26999 2008-05-31 14:35:42Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,10 +12,12 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
@@ -23,7 +25,7 @@
 #define INC_EVENTS_H
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include <X11/X.h>
@@ -36,7 +38,7 @@
 #include "event_filter.h"
 
 void                     initMenuEventWin                       (void);
-eventFilterStatus        xfwm4_event_filter                     (XEvent *, 
+eventFilterStatus        xfwm4_event_filter                     (XEvent *,
                                                                  gpointer);
 void                     initGtkCallbacks                       (ScreenInfo *);
 
