@@ -1,4 +1,4 @@
-/*      $Id: focus.c 28362 2008-10-22 19:58:39Z olivier $
+/*      $Id: focus.c 27081 2008-06-19 07:18:24Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,15 +12,17 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/X.h>
@@ -57,9 +59,26 @@ static Client *client_focus  = NULL;
 static Client *pending_focus = NULL;
 static Client *user_focus    = NULL;
 static Client *last_ungrab   = NULL;
+static Client *delayed_focus = NULL;
+static guint focus_timeout   = 0;
+
+#if 0
+static void
+clientDumpList (ScreenInfo *screen_info)
+{
+    int i;
+    Client *c;
+
+    g_print ("Dumping client list\n");
+    for (c = screen_info->clients, i = 0; (c) && (i < screen_info->client_count); c = c->next, i++)
+    {
+        g_print ("    [%i] 0x%lx - %s\n", i, c->window, c->name);
+    }
+}
+#endif
 
 static ClientPair
-clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, Client * exclude)
+clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, GList * exclude_list)
 {
     ClientPair top_client;
     Client *c;
@@ -79,9 +98,13 @@ clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, Client * exclude)
             continue;
         }
 
-        if (!exclude || (c != exclude))
+        if (!g_list_find (exclude_list, (gconstpointer) c))
         {
-            if ((c->win_layer <= layer) && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
+            if (c->win_layer > layer)
+            {
+                break;
+            }
+            else if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
             {
                 if (clientSelectMask (c, 0, WINDOW_REGULAR_FOCUSABLE))
                 {
@@ -89,10 +112,6 @@ clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, Client * exclude)
                 }
                 top_client.highest = c;
             }
-            else if (c->win_layer > layer)
-            {
-                break;
-            }
         }
     }
 
@@ -100,7 +119,7 @@ clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, Client * exclude)
 }
 
 void
-clientFocusTop (ScreenInfo *screen_info, int layer, guint32 timestamp)
+clientFocusTop (ScreenInfo *screen_info, int layer, Time timestamp)
 {
     ClientPair top_client;
     DisplayInfo *display_info;
@@ -145,7 +164,7 @@ clientFocusNew(Client * c)
     }
     else if ((client_focus) && (prevent_focus_stealing))
     {
-        if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STARTUP_TIME) && (c->user_time == CurrentTime))
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STARTUP_TIME) && (c->user_time == (Time) CurrentTime))
         {
             TRACE ("Given startup time is 0, not focusing");
             give_focus = FALSE;
@@ -174,6 +193,7 @@ clientFocusNew(Client * c)
         clientSetFocus (screen_info, c,
                         myDisplayGetCurrentTime (display_info),
                         FOCUS_IGNORE_MODAL);
+        clientPassGrabMouseButton (c);
     }
     else
     {
@@ -285,7 +305,7 @@ clientGetPrevious (Client * c, int mask)
 }
 
 void
-clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
+clientPassFocus (ScreenInfo *screen_info, Client *c, GList *exclude_list)
 {
     DisplayInfo *display_info;
     ClientPair top_most;
@@ -303,12 +323,12 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
     new_focus = NULL;
     current_focus = client_focus;
     c2 = NULL;
-
+#if 1
     if (pending_focus)
     {
         current_focus = pending_focus;
     }
-
+#endif
     if ((c || current_focus) && (c != current_focus))
     {
         return;
@@ -320,17 +340,19 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
     }
 
     display_info = screen_info->display_info;
-    top_most = clientGetTopMostFocusable (screen_info, look_in_layer, exclude);
+    top_most = clientGetTopMostFocusable (screen_info, look_in_layer, exclude_list);
+#if 0
     if (screen_info->params->click_to_focus)
     {
         if (c)
         {
             if (clientIsModal (c))
             {
-                /* If the window is a modal, send focus back to its parent window.
-                   Modals are transients, and we aren't interested in modal
-                   for group, so it safe to use clientGetTransient because
-                   it's really what we want...
+                /* If the window is a modal, send focus back to its parent
+                 * window.
+                 * Modals are transients, and we aren't interested in modal
+                 * for group, so it safe to use clientGetTransient because
+                 * it's really what we want...
                  */
                 c2 = clientGetTransient (c);
                 if (c2 && FLAG_TEST(c2->xfwm_flags, XFWM_FLAG_VISIBLE))
@@ -350,9 +372,12 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
             }
         }
     }
-    else if (XQueryPointer (myScreenGetXDisplay (screen_info), screen_info->xroot, &dr, &window, &rx, &ry, &wx, &wy, &mask))
+    else
+#endif
+    if (!(screen_info->params->click_to_focus) &&
+        XQueryPointer (myScreenGetXDisplay (screen_info), screen_info->xroot, &dr, &window, &rx, &ry, &wx, &wy, &mask))
     {
-        new_focus = clientAtPosition (screen_info, rx, ry, exclude);
+        new_focus = clientAtPosition (screen_info, rx, ry, exclude_list);
     }
     if (!new_focus)
     {
@@ -394,7 +419,7 @@ clientSortRing(Client *c)
     ScreenInfo *screen_info;
 
     g_return_if_fail (c != NULL);
-    TRACE ("Sorting...");
+    TRACE ("entering clientSortRing");
 
     screen_info = c->screen_info;
     if ((screen_info->client_count > 2) && (c != screen_info->clients))
@@ -404,18 +429,66 @@ clientSortRing(Client *c)
 
         c->prev = screen_info->clients->prev;
         c->next = screen_info->clients;
+
         screen_info->clients->prev->next = c;
         screen_info->clients->prev = c;
     }
     screen_info->clients = c;
 }
 
+void
+clientSetLast(Client *c)
+{
+    ScreenInfo *screen_info;
+
+    g_return_if_fail (c != NULL);
+    TRACE ("entering clientSetLast");
+
+    screen_info = c->screen_info;
+    if (screen_info->client_count > 2)
+    {
+        if (screen_info->clients == c)
+        {
+            screen_info->clients = screen_info->clients->next;
+        }
+        else
+        {
+            c->prev->next = c->next;
+            c->next->prev = c->prev;
+
+            c->prev = screen_info->clients->prev;
+            c->next = screen_info->clients;
+
+            screen_info->clients->prev->next = c;
+            screen_info->clients->prev = c;
+        }
+    }
+}
+
+static void
+clientFocusNone (ScreenInfo *screen_info, Client *previous, Time timestamp)
+{
+    TRACE ("setting focus to none");
+
+    pending_focus = NULL;
+
+    if (previous)
+    {
+        frameQueueDraw (previous, FALSE);
+        if (previous->screen_info != screen_info)
+        {
+            clientSetNetActiveWindow (previous->screen_info, NULL, timestamp);
+        }
+    }
+    clientSetNetActiveWindow (screen_info, NULL, timestamp);
+    XSetInputFocus (myScreenGetXDisplay (screen_info), screen_info->xfwm4_win, RevertToPointerRoot, timestamp);
+}
+
 void
 clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
 {
     DisplayInfo *display_info;
     Client *c2;
-    unsigned long data[2];
 
     TRACE ("entering clientUpdateFocus");
 
@@ -454,37 +527,30 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         }
         if (FLAG_TEST(c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
         {
-            TRACE ("Un-setting WM_STATE_DEMANDS_ATTENTION flag on \"%s\" (0x%lx)", c->name, c->window); 
+            TRACE ("Un-setting WM_STATE_DEMANDS_ATTENTION flag on \"%s\" (0x%lx)", c->name, c->window);
             FLAG_UNSET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
             clientSetNetState (c);
         }
-        data[0] = c->window;
         clientAdjustFullscreenLayer (c, TRUE);
-        frameDraw (c, FALSE);
-    }
-    else
-    {
-        data[0] = None;
+        frameQueueDraw (c, FALSE);
+        clientUpdateOpacity (c);
+
     }
     if (c2)
     {
         if (c)
         {
             clientAdjustFullscreenLayer (c2, FALSE);
-            /* clientRaise (c, None); */
         }
-        frameDraw (c2, FALSE);
+        frameQueueDraw (c2, FALSE);
+        clientUpdateOpacity (c2);
     }
-    data[1] = None;
-    XChangeProperty (display_info->dpy, screen_info->xroot,
-                     display_info->atoms[NET_ACTIVE_WINDOW], XA_WINDOW, 32,
-                     PropModeReplace, (unsigned char *) data, 2);
-
-    clientUpdateOpacity (screen_info, c);
+    clientSetNetActiveWindow (screen_info, c, 0);
+    clientClearDelayedFocus ();
 }
 
 void
-clientSetFocus (ScreenInfo *screen_info, Client * c, guint32 timestamp, unsigned short flags)
+clientSetFocus (ScreenInfo *screen_info, Client *c, Time timestamp, unsigned short flags)
 {
     DisplayInfo *display_info;
     Client *c2;
@@ -509,7 +575,7 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, guint32 timestamp, unsigned
         user_focus = c;
         if (FLAG_TEST(c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
         {
-            TRACE ("Un-setting WM_STATE_DEMANDS_ATTENTION flag on \"%s\" (0x%lx)", c->name, c->window); 
+            TRACE ("Un-setting WM_STATE_DEMANDS_ATTENTION flag on \"%s\" (0x%lx)", c->name, c->window);
             FLAG_UNSET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
             clientSetNetState (c);
         }
@@ -538,12 +604,17 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, guint32 timestamp, unsigned
             {
                 XSetInputFocus (myScreenGetXDisplay (screen_info), c->window, RevertToPointerRoot, timestamp);
             }
-            clientUpdateOpacity (screen_info, c);
         }
-        else if (!client_focus)
+        else
         {
-            /* Hack to prevent loosing focus when all remaining windows won't accept focus, see bug #1853 */
-            XSetInputFocus (myScreenGetXDisplay (screen_info), screen_info->xfwm4_win, RevertToPointerRoot, timestamp);
+            /*
+             * If we are relying only on the client application to take focus, we need to set the focus
+             * explicitely on our own fallback window otherwise there is a race condition between the
+             * application and the window manager. If the application does not take focus before the
+             * the previously focused window is unmapped (when iconifying or closing for example), the focus
+             * will be reverted to the root window and focus transition will fail.
+             */
+            clientFocusNone (screen_info, c2, timestamp);
         }
 
         if (FLAG_TEST(c->wm_flags, WM_FLAG_TAKEFOCUS))
@@ -554,24 +625,11 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, guint32 timestamp, unsigned
     }
     else
     {
-        unsigned long data[2];
-
         TRACE ("setting focus to none");
 
-        data[0] = data[1] = None;
         client_focus = NULL;
-        pending_focus = NULL;
-
-        if (c2)
-        {
-            frameDraw (c2, FALSE);
-            XChangeProperty (clientGetXDisplay (c2), c2->screen_info->xroot, display_info->atoms[NET_ACTIVE_WINDOW], XA_WINDOW, 32,
-                             PropModeReplace, (unsigned char *) data, 2);
-        }
-        XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, display_info->atoms[NET_ACTIVE_WINDOW], XA_WINDOW, 32,
-                         PropModeReplace, (unsigned char *) data, 2);
-        XSetInputFocus (myScreenGetXDisplay (screen_info), screen_info->xfwm4_win, RevertToPointerRoot, timestamp);
-        clientUpdateOpacity (screen_info, c);
+        clientFocusNone (screen_info, c2, timestamp);
+        clientClearDelayedFocus ();
     }
 }
 
@@ -731,17 +789,16 @@ clientPassGrabMouseButton (Client * c)
         return;
     }
 
-    TRACE ("ungrabing buttons for client \"%s\" (0x%lx)", c->name, c->window);
     if (last_ungrab == c)
     {
         return;
     }
 
+    TRACE ("ungrabing buttons for client \"%s\" (0x%lx)", c->name, c->window);
     if (last_ungrab)
     {
         clientGrabMouseButton (last_ungrab);
     }
-
     clientUngrabMouseButton (c);
     last_ungrab = c;
 }
@@ -757,3 +814,51 @@ clientClearLastUngrab (void)
 {
     last_ungrab = NULL;
 }
+
+static gboolean
+delayed_focus_cb (gpointer data)
+{
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+
+    TRACE ("entering delayed_focus_cb");
+    g_return_val_if_fail (delayed_focus != NULL, FALSE);
+
+    screen_info = delayed_focus->screen_info;
+    display_info = screen_info->display_info;
+    clientSetFocus (screen_info, delayed_focus, myDisplayGetCurrentTime (display_info), NO_FOCUS_FLAG);
+    focus_timeout = 0;
+    delayed_focus = NULL;
+
+    return (FALSE);
+}
+
+void
+clientClearDelayedFocus (void)
+{
+    if(focus_timeout)
+    {
+        g_source_remove (focus_timeout);
+        focus_timeout = 0;
+    }
+    delayed_focus = NULL;
+}
+
+void
+clientAddDelayedFocus (Client *c)
+{
+    ScreenInfo *screen_info;
+
+    screen_info = c->screen_info;
+    delayed_focus = c;
+    focus_timeout = g_timeout_add_full (G_PRIORITY_DEFAULT,
+                                        screen_info->params->focus_delay,
+                                        (GSourceFunc) delayed_focus_cb,
+                                        NULL, NULL);
+}
+
+Client *
+clientGetDelayedFocus (void)
+{
+    return delayed_focus;
+}
