@@ -1,4 +1,4 @@
-/*      $Id: hints.c 28362 2008-10-22 19:58:39Z olivier $
+/*      $Id: hints.c 27357 2008-07-21 09:14:12Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,16 +12,18 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/X.h>
@@ -397,7 +399,7 @@ setGnomeProtocols (DisplayInfo *display_info, Window root, Window w)
 void
 setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
 {
-    Atom atoms[ATOM_COUNT];
+    Atom atoms[64];
     unsigned long data[1];
     int i;
 
@@ -412,7 +414,6 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
     atoms[i++] = display_info->atoms[NET_DESKTOP_NAMES];
     atoms[i++] = display_info->atoms[NET_DESKTOP_VIEWPORT];
     atoms[i++] = display_info->atoms[NET_FRAME_EXTENTS];
-    atoms[i++] = display_info->atoms[NET_MOVERESIZE_WINDOW];
     atoms[i++] = display_info->atoms[NET_NUMBER_OF_DESKTOPS];
     atoms[i++] = display_info->atoms[NET_REQUEST_FRAME_EXTENTS];
     atoms[i++] = display_info->atoms[NET_SHOWING_DESKTOP];
@@ -451,6 +452,7 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
     atoms[i++] = display_info->atoms[NET_WM_SYNC_REQUEST];
     atoms[i++] = display_info->atoms[NET_WM_SYNC_REQUEST_COUNTER];
     atoms[i++] = display_info->atoms[NET_WM_USER_TIME];
+    atoms[i++] = display_info->atoms[NET_WM_USER_TIME_WINDOW];
     atoms[i++] = display_info->atoms[NET_WM_WINDOW_TYPE];
     atoms[i++] = display_info->atoms[NET_WM_WINDOW_TYPE_DESKTOP];
     atoms[i++] = display_info->atoms[NET_WM_WINDOW_TYPE_DIALOG];
@@ -464,7 +466,7 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
     atoms[i++] = display_info->atoms[NET_STARTUP_ID];
 #endif
-    g_assert (i < ATOM_COUNT);
+    g_assert (i < 64);
     data[0] = check_win;
     XChangeProperty (display_info->dpy, root, display_info->atoms[NET_SUPPORTED],
                      XA_ATOM, 32, PropModeReplace, (unsigned char *) atoms, i);
@@ -704,6 +706,7 @@ getUTF8StringData (DisplayInfo *display_info, Window w, int atom_id, gchar **str
     unsigned long n_items;
 
     g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
+
     TRACE ("entering getUTF8StringData");
 
     *str_p = NULL;
@@ -737,6 +740,7 @@ getUTF8String (DisplayInfo *display_info, Window w, int atom_id, gchar **str_p,
     char *xstr;
 
     g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
+
     TRACE ("entering getUTF8String");
 
     if (!getUTF8StringData (display_info, w, atom_id, &xstr, length))
@@ -811,6 +815,7 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, gchar ***st
         if (g_utf8_validate (ptr, -1, NULL))
         {
             retval[i] = internal_utf8_strndup (ptr, MAX_STR_LENGTH);
+            xfce_utf8_remove_controls((gchar *) retval[i], -1, NULL);
         }
         else
         {
@@ -825,6 +830,40 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, gchar ***st
     return TRUE;
 }
 
+gboolean
+getWindowProp (DisplayInfo *display_info, Window window, int atom_id, Window *w)
+{
+    Atom type;
+    int format;
+    unsigned long nitems;
+    unsigned long bytes_after;
+    unsigned char *prop;
+
+    TRACE ("entering getWindowProp");
+
+    g_return_val_if_fail (window != None, FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
+
+    *w = None;
+    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[atom_id],
+                            0L, 1L, FALSE, XA_WINDOW, &type, &format, &nitems,
+                            &bytes_after, (unsigned char **) &prop) == Success)
+    {
+        if (prop)
+        {
+            *w = *((Window *) prop);
+            XFree (prop);
+        }
+        if (!prop || !check_type_and_format (32, XA_WINDOW, -1, format, type))
+        {
+            *w = None;
+            return FALSE;
+        }
+    }
+
+    return TRUE;
+}
+
 gboolean
 getClientMachine (DisplayInfo *display_info, Window w, gchar **machine)
 {
@@ -836,6 +875,7 @@ getClientMachine (DisplayInfo *display_info, Window w, gchar **machine)
     g_return_val_if_fail (machine != NULL, FALSE);
     *machine = NULL;
     g_return_val_if_fail (w != None, FALSE);
+    g_return_val_if_fail (display_info != NULL, FALSE);
 
     status = FALSE;
     str = get_text_property (display_info, w, display_info->atoms[WM_CLIENT_MACHINE]);
@@ -874,6 +914,7 @@ getWindowName (DisplayInfo *display_info, Window w, gchar **title)
     {
         name = internal_utf8_strndup (str, MAX_STR_LENGTH);
         *title = create_name_with_host (display_info, name, machine);
+        xfce_utf8_remove_controls(*title, -1, NULL);
         g_free (name);
         XFree (str);
         status = TRUE;
@@ -918,36 +959,17 @@ Window
 getClientLeader (DisplayInfo *display_info, Window window)
 {
     Window client_leader;
-    Atom actual_type;
-    int actual_format;
-    unsigned long nitems;
-    unsigned long bytes_after;
-    unsigned char *prop;
-
     TRACE ("entering getClientLeader");
 
     g_return_val_if_fail (window != None, None);
 
     client_leader = None;
-    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[WM_CLIENT_LEADER],
-                            0L, 1L, FALSE, AnyPropertyType, &actual_type, &actual_format, &nitems,
-                            &bytes_after, (unsigned char **) &prop) == Success)
-    {
-        if ((prop) && (actual_type == XA_WINDOW) && (actual_format == 32)
-            && (nitems == 1) && (bytes_after == 0))
-        {
-            client_leader = *((Window *) prop);
-        }
-        if (prop)
-        {
-            XFree (prop);
-        }
-    }
+    getWindowProp (display_info, window, WM_CLIENT_LEADER, &client_leader);
     return client_leader;
 }
 
 gboolean
-getNetWMUserTime (DisplayInfo *display_info, Window window, guint32 *time)
+getNetWMUserTime (DisplayInfo *display_info, Window window, Time *time)
 {
     Atom actual_type;
     int actual_format;
@@ -966,16 +988,32 @@ getNetWMUserTime (DisplayInfo *display_info, Window window, guint32 *time)
         if ((data) && (actual_type == XA_CARDINAL)
             && (nitems == 1) && (bytes_after == 0))
         {
-            *time = *((guint32 *) data);
+            *time = *((long *) data);
             XFree (data);
             return TRUE;
         }
     }
-    *time = 0;
+    *time = (Time) 0;
 
     return FALSE;
 }
 
+Window
+getNetWMUserTimeWindow (DisplayInfo *display_info, Window window)
+{
+    Window user_time_win;
+    TRACE ("entering getNetWMUserTimeWindow");
+
+    g_return_val_if_fail (window != None, None);
+
+    user_time_win = None;
+    if (getWindowProp (display_info, window, NET_WM_USER_TIME_WINDOW, &user_time_win))
+    {
+        return user_time_win;
+    }
+    return window;
+}
+
 gboolean
 getClientID (DisplayInfo *display_info, Window window, gchar **client_id)
 {
@@ -988,7 +1026,7 @@ getClientID (DisplayInfo *display_info, Window window, gchar **client_id)
     *client_id = NULL;
     g_return_val_if_fail (window != None, FALSE);
 
-    if ((id = getClientLeader (display_info, window)))
+    if (getWindowProp (display_info, window, WM_CLIENT_LEADER, &id) && (id != None))
     {
         if (XGetTextProperty (display_info->dpy, id, &tp, display_info->atoms[SM_CLIENT_ID]))
         {
@@ -1016,7 +1054,7 @@ getWindowCommand (DisplayInfo *display_info, Window window, char ***argv, int *a
     {
         return TRUE;
     }
-    if ((id = getClientLeader (display_info, window)))
+    if (getWindowProp (display_info, window, WM_CLIENT_LEADER, &id) && (id != None))
     {
         if (XGetCommand (display_info->dpy, id, argv, argc) && (*argc > 0))
         {
@@ -1127,7 +1165,7 @@ gboolean
 setXAtomManagerOwner (DisplayInfo *display_info, Atom atom, Window root, Window w)
 {
     XClientMessageEvent ev;
-    guint32 server_time;
+    Time server_time;
     int status;
 
     g_return_val_if_fail (root != None, FALSE);
@@ -1181,27 +1219,27 @@ updateXserverTime (DisplayInfo *display_info)
                      8, PropModeReplace, (unsigned char *) &c, 1);
 }
 
-guint32
+Time
 getXServerTime (DisplayInfo *display_info)
 {
     ScreenInfo *screen_info;
     XEvent xevent;
-    guint32 timestamp;
+    Time timestamp;
 
-    g_return_val_if_fail (display_info, CurrentTime);
+    g_return_val_if_fail (display_info, (Time) CurrentTime);
     timestamp = myDisplayGetCurrentTime (display_info);
-    if (timestamp == CurrentTime)
+    if (timestamp == (Time) CurrentTime)
     {
         screen_info = myDisplayGetDefaultScreen (display_info);
-        g_return_val_if_fail (screen_info,  CurrentTime);
+        g_return_val_if_fail (screen_info,  (Time) CurrentTime);
 
         TRACE ("getXServerTime: Using X server roundtrip");
         updateXserverTime (display_info);
         XWindowEvent (display_info->dpy, display_info->timestamp_win, PropertyChangeMask, &xevent);
-        timestamp = myDisplayUpdateCurrentTime (display_info, &xevent);
+        timestamp = (Time) myDisplayUpdateCurrentTime (display_info, &xevent);
     }
 
-    TRACE ("getXServerTime gives timestamp=%u", timestamp);
+    TRACE ("getXServerTime gives timestamp=%u", (unsigned int) timestamp);
     return timestamp;
 }
 
@@ -1310,44 +1348,3 @@ getWindowStartupId (DisplayInfo *display_info, Window w, gchar **startup_id)
     return FALSE;
 }
 #endif
-
-#ifdef HAVE_XSYNC
-gboolean
-getXSyncCounter (DisplayInfo *display_info, Window window, XSyncCounter *counter)
-{
-    long val;
-
-    g_return_val_if_fail (window != None, FALSE);
-    g_return_val_if_fail (counter != NULL, FALSE);
-    TRACE ("entering getXSyncCounter");
-
-    val = 0;
-    if (getHint (display_info, window, NET_WM_SYNC_REQUEST_COUNTER, &val))
-    {
-        *counter = (XSyncCounter) val;
-        return TRUE;
-    }
-
-    return FALSE;
-}
-
-void
-sendXSyncRequest (DisplayInfo *display_info, Window window, XSyncValue value)
-{
-    XClientMessageEvent xev;
-
-    g_return_if_fail (window != None);
-    TRACE ("entering getXSyncCounter");
-
-    xev.type = ClientMessage;
-    xev.window = window;
-    xev.message_type = display_info->atoms[WM_PROTOCOLS];
-    xev.format = 32;
-    xev.data.l[0] = (long) display_info->atoms[NET_WM_SYNC_REQUEST];
-    xev.data.l[1] = (long) myDisplayGetCurrentTime (display_info);
-    xev.data.l[2] = (long) XSyncValueLow32 (value);
-    xev.data.l[3] = (long) XSyncValueHigh32 (value);
-    xev.data.l[4] = (long) 0L;
-    XSendEvent (display_info->dpy, window, FALSE, NoEventMask, (XEvent *) &xev);
-}
-#endif /* HAVE_XSYNC */
