@@ -1,4 +1,4 @@
-/*      $Id: hints.h 28362 2008-10-22 19:58:39Z olivier $
+/*      $Id: hints.h 27356 2008-07-20 21:22:15Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,11 +12,13 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
@@ -24,7 +26,7 @@
 #define INC_HINTS_H
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include <X11/X.h>
@@ -88,6 +90,7 @@
 #define WIN_LAYER_ONTOP                         6
 #define WIN_LAYER_DOCK                          8
 #define WIN_LAYER_ABOVE_DOCK                    10
+#define WIN_LAYER_FULLSCREEN                    12
 
 #define NET_WM_MOVERESIZE_SIZE_TOPLEFT          0
 #define NET_WM_MOVERESIZE_SIZE_TOP              1
@@ -149,152 +152,146 @@ typedef struct
 }
 PropMwmHints;
 
-unsigned long            getWMState                             (DisplayInfo *, 
+unsigned long            getWMState                             (DisplayInfo *,
                                                                  Window);
-void                     setWMState                             (DisplayInfo *, 
-                                                                 Window, 
+void                     setWMState                             (DisplayInfo *,
+                                                                 Window,
                                                                  unsigned long);
-PropMwmHints            *getMotifHints                          (DisplayInfo *, 
+PropMwmHints            *getMotifHints                          (DisplayInfo *,
                                                                  Window);
-unsigned int             getWMProtocols                         (DisplayInfo *, 
+unsigned int             getWMProtocols                         (DisplayInfo *,
                                                                  Window);
-gboolean                 getHint                                (DisplayInfo *, 
-                                                                 Window, 
-                                                                 int, 
+gboolean                 getHint                                (DisplayInfo *,
+                                                                 Window,
+                                                                 int,
                                                                  long *);
-void                     setHint                                (DisplayInfo *, 
-                                                                 Window, 
-                                                                 int, 
+void                     setHint                                (DisplayInfo *,
+                                                                 Window,
+                                                                 int,
                                                                  long);
-void                     getDesktopLayout                       (DisplayInfo *, 
-                                                                 Window, 
-                                                                 int, 
+void                     getDesktopLayout                       (DisplayInfo *,
+                                                                 Window,
+                                                                 int,
                                                                  NetWmDesktopLayout *);
-void                     getGnomeDesktopMargins                 (DisplayInfo *, 
-                                                                 Window, 
+void                     getGnomeDesktopMargins                 (DisplayInfo *,
+                                                                 Window,
                                                                  int *);
-void                     setGnomeProtocols                      (DisplayInfo *, 
-                                                                 Window, 
+void                     setGnomeProtocols                      (DisplayInfo *,
+                                                                 Window,
                                                                  Window);
-void                     setNetSupportedHint                    (DisplayInfo *, 
-                                                                 Window, 
+void                     setNetSupportedHint                    (DisplayInfo *,
+                                                                 Window,
                                                                  Window);
-gboolean                 getAtomList                            (DisplayInfo *, 
-                                                                 Window, 
-                                                                 int, 
-                                                                 Atom **, 
+gboolean                 getAtomList                            (DisplayInfo *,
+                                                                 Window,
+                                                                 int,
+                                                                 Atom **,
                                                                  int *);
-gboolean                 getCardinalList                        (DisplayInfo *, 
-                                                                 Window, 
-                                                                 int, 
-                                                                 unsigned long **, 
+gboolean                 getCardinalList                        (DisplayInfo *,
+                                                                 Window,
+                                                                 int,
+                                                                 unsigned long **,
                                                                  int *);
-void                     setNetWorkarea                         (DisplayInfo *, 
-                                                                 Window, 
-                                                                 int, 
-                                                                 int, 
-                                                                 int, 
+void                     setNetWorkarea                         (DisplayInfo *,
+                                                                 Window,
+                                                                 int,
+                                                                 int,
+                                                                 int,
                                                                  int *);
-void                     setNetFrameExtents                     (DisplayInfo *, 
-                                                                 Window, 
-                                                                 int, 
-                                                                 int, 
-                                                                 int, 
+void                     setNetFrameExtents                     (DisplayInfo *,
+                                                                 Window,
+                                                                 int,
+                                                                 int,
+                                                                 int,
                                                                  int);
-void                     initNetDesktopInfo                     (DisplayInfo *, 
-                                                                 Window, 
-                                                                 int, 
-                                                                 int, 
+void                     initNetDesktopInfo                     (DisplayInfo *,
+                                                                 Window,
+                                                                 int,
+                                                                 int,
                                                                  int);
-void                     setUTF8StringHint                      (DisplayInfo *, 
-                                                                 Window, 
-                                                                 int, 
+void                     setUTF8StringHint                      (DisplayInfo *,
+                                                                 Window,
+                                                                 int,
                                                                  const gchar *);
-void                     getTransientFor                        (DisplayInfo *, 
-                                                                 Window, 
-                                                                 Window, 
+void                     getTransientFor                        (DisplayInfo *,
+                                                                 Window,
+                                                                 Window,
                                                                  Window *);
-gboolean                 getWindowName                          (DisplayInfo *, 
-                                                                 Window, 
+gboolean                 getWindowName                          (DisplayInfo *,
+                                                                 Window,
                                                                  gchar **);
-gboolean                 getUTF8String                          (DisplayInfo *, 
-                                                                 Window, 
-                                                                 int, 
-                                                                 gchar **, 
+gboolean                 getUTF8String                          (DisplayInfo *,
+                                                                 Window,
+                                                                 int,
+                                                                 gchar **,
                                                                  int *);
-gboolean                 getUTF8StringList                      (DisplayInfo *, 
-                                                                 Window, 
-                                                                 int, 
-                                                                 gchar ***, 
+gboolean                 getUTF8StringList                      (DisplayInfo *,
+                                                                 Window,
+                                                                 int,
+                                                                 gchar ***,
                                                                  int *);
-gboolean                 getClientMachine                       (DisplayInfo *, 
-                                                                 Window, 
+gboolean                 getWindowProp                          (DisplayInfo *,
+                                                                 Window,
+                                                                 int,
+                                                                 Window *);
+gboolean                 getClientMachine                       (DisplayInfo *,
+                                                                 Window,
                                                                  gchar **);
-gboolean                 getClientMachine                       (DisplayInfo *, 
-                                                                 Window, 
+gboolean                 getWindowRole                          (DisplayInfo *,
+                                                                 Window,
                                                                  gchar **);
-gboolean                 getWindowRole                          (DisplayInfo *, 
-                                                                 Window, 
-                                                                 gchar **);
-Window                   getClientLeader                        (DisplayInfo *, 
+Window                   getClientLeader                        (DisplayInfo *,
                                                                  Window);
-gboolean                 getNetWMUserTime                       (DisplayInfo *, 
-                                                                 Window, 
-                                                                 guint32 *);
-gboolean                 getClientID                            (DisplayInfo *, 
-                                                                 Window, 
+gboolean                 getNetWMUserTime                       (DisplayInfo *,
+                                                                 Window,
+                                                                 Time *);
+Window                   getNetWMUserTimeWindow                 (DisplayInfo *,
+                                                                 Window);
+gboolean                 getClientID                            (DisplayInfo *,
+                                                                 Window,
                                                                  gchar **);
-gboolean                 getWindowCommand                       (DisplayInfo *, 
-                                                                 Window, 
-                                                                 char ***, 
+gboolean                 getWindowCommand                       (DisplayInfo *,
+                                                                 Window,
+                                                                 char ***,
                                                                  int *);
-gboolean                 getKDEIcon                             (DisplayInfo *, 
-                                                                 Window, 
-                                                                 Pixmap *, 
+gboolean                 getKDEIcon                             (DisplayInfo *,
+                                                                 Window,
+                                                                 Pixmap *,
                                                                  Pixmap *);
-gboolean                 getRGBIconData                         (DisplayInfo *, 
-                                                                 Window, 
-                                                                 unsigned long **, 
+gboolean                 getRGBIconData                         (DisplayInfo *,
+                                                                 Window,
+                                                                 unsigned long **,
                                                                  unsigned long *);
-gboolean                 getOpacity                             (DisplayInfo *, 
-                                                                 Window, 
+gboolean                 getOpacity                             (DisplayInfo *,
+                                                                 Window,
                                                                  guint *);
-gboolean                 getOpacityLock                         (DisplayInfo *, 
+gboolean                 getOpacityLock                         (DisplayInfo *,
                                                                  Window);
-gboolean                 setXAtomManagerOwner                   (DisplayInfo *, 
-                                                                 Atom, 
-                                                                 Window, 
+gboolean                 setXAtomManagerOwner                   (DisplayInfo *,
+                                                                 Atom,
+                                                                 Window,
                                                                  Window);
-gboolean                 setAtomIdManagerOwner                  (DisplayInfo *, 
-                                                                 int, 
-                                                                 Window , 
+gboolean                 setAtomIdManagerOwner                  (DisplayInfo *,
+                                                                 int,
+                                                                 Window ,
                                                                  Window);
 void                     updateXserverTime                      (DisplayInfo *);
-guint32                  getXServerTime                         (DisplayInfo *);
+Time                     getXServerTime                         (DisplayInfo *);
 
 #ifdef ENABLE_KDE_SYSTRAY_PROXY
-gboolean                 checkKdeSystrayWindow                  (DisplayInfo *, 
+gboolean                 checkKdeSystrayWindow                  (DisplayInfo *,
                                                                  Window);
-void                     sendSystrayReqDock                     (DisplayInfo *, 
-                                                                 Window, 
+void                     sendSystrayReqDock                     (DisplayInfo *,
+                                                                 Window,
                                                                  Window);
-Window                   getSystrayWindow                       (DisplayInfo *, 
+Window                   getSystrayWindow                       (DisplayInfo *,
                                                                  Atom);
 #endif
 
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
-gboolean                 getWindowStartupId                     (DisplayInfo *, 
-                                                                 Window, 
+gboolean                 getWindowStartupId                     (DisplayInfo *,
+                                                                 Window,
                                                                  char **);
 #endif
 
-#ifdef HAVE_XSYNC
-gboolean                 getXSyncCounter                        (DisplayInfo *, 
-                                                                 Window, 
-                                                                 XSyncCounter *);
-void                     sendXSyncRequest                       (DisplayInfo *, 
-                                                                 Window, 
-                                                                 XSyncValue);
-#endif /* HAVE_XSYNC */
-
 #endif /* INC_HINTS_H */
