@@ -1,4 +1,4 @@
-/*      $Id: keyboard.c 25673 2007-05-02 21:08:49Z olivier $
+/*      $Id: keyboard.c 27421 2008-08-04 20:28:44Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,15 +12,17 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/Xlib.h>
@@ -38,8 +40,31 @@ unsigned int ScrollLockMask;
 unsigned int SuperMask;
 unsigned int HyperMask;
 
+static gboolean
+getKeycode (Display *dpy, const char *str, KeyCode *keycode)
+{
+    unsigned int value;
+    KeySym keysym;
+
+    keysym = XStringToKeysym (str);
+    if (keysym == NoSymbol)
+    {
+        if (sscanf (str, "0x%X", (unsigned int *) &value) != 1)
+        {
+            *keycode = 0;
+            return FALSE;
+        }
+        *keycode = (KeyCode) value;
+    }
+    else
+    {
+        *keycode = XKeysymToKeycode (dpy, keysym);
+    }
+    return TRUE;
+}
+
 int
-getModifierMap (char *str)
+getModifierMap (const char *str)
 {
     gchar *tmp;
     int map;
@@ -97,7 +122,7 @@ getModifierMap (char *str)
 }
 
 void
-parseKeyString (Display * dpy, MyKey * key, char *str)
+parseKeyString (Display * dpy, MyKey * key, const char *str)
 {
     char *k;
 
@@ -120,57 +145,71 @@ parseKeyString (Display * dpy, MyKey * key, char *str)
     if (k)
     {
         /* There is a modifier */
-        key->keycode = XKeysymToKeycode (dpy, XStringToKeysym (++k));
+        getKeycode (dpy, ++k, &key->keycode);
         key->modifier = getModifierMap (str);
     }
     else
     {
-        key->keycode = XKeysymToKeycode (dpy, XStringToKeysym (str));
+        getKeycode (dpy, str, &key->keycode);
         key->modifier = 0;
     }
 }
 
-void
+gboolean
 grabKey (Display * dpy, MyKey * key, Window w)
 {
+    gboolean status;
+
     TRACE ("entering grabKey");
 
+    status = GrabSuccess;
     if (key->keycode)
     {
         if (key->modifier == 0)
         {
-            XGrabKey (dpy, key->keycode, AnyModifier, w, FALSE,
-                GrabModeAsync, GrabModeSync);
+            status |=
+                XGrabKey (dpy, key->keycode, AnyModifier, w,
+                                        TRUE, GrabModeAsync, GrabModeSync);
         }
         else
         {
             /* Here we grab all combinations of well known modifiers */
-            XGrabKey (dpy, key->keycode,
-                key->modifier, w, FALSE,
-                GrabModeAsync, GrabModeSync);
-            XGrabKey (dpy, key->keycode,
-                key->modifier | ScrollLockMask, w,
-                FALSE, GrabModeAsync, GrabModeSync);
-            XGrabKey (dpy, key->keycode,
-                key->modifier | NumLockMask, w,
-                FALSE, GrabModeAsync, GrabModeSync);
-            XGrabKey (dpy, key->keycode,
-                key->modifier | LockMask, w,
-                FALSE, GrabModeAsync, GrabModeSync);
-            XGrabKey (dpy, key->keycode,
-                key->modifier | ScrollLockMask | NumLockMask, w, FALSE,
-                GrabModeAsync, GrabModeSync);
-            XGrabKey (dpy, key->keycode,
-                key->modifier | ScrollLockMask | LockMask, w, FALSE,
-                GrabModeAsync, GrabModeSync);
-            XGrabKey (dpy, key->keycode,
-                key->modifier | LockMask | NumLockMask, w, FALSE,
-                GrabModeAsync, GrabModeSync);
-            XGrabKey (dpy, key->keycode,
-                key->modifier | ScrollLockMask | LockMask | NumLockMask,
-                w, FALSE, GrabModeAsync, GrabModeSync);
+            status |=
+                XGrabKey (dpy, key->keycode,
+                                        key->modifier, w,
+                                        TRUE, GrabModeAsync, GrabModeSync);
+            status |=
+                XGrabKey (dpy, key->keycode,
+                                        key->modifier | ScrollLockMask, w,
+                                        TRUE, GrabModeAsync, GrabModeSync);
+            status |=
+                XGrabKey (dpy, key->keycode,
+                                        key->modifier | NumLockMask, w,
+                                        TRUE, GrabModeAsync, GrabModeSync);
+            status |=
+                XGrabKey (dpy, key->keycode,
+                                        key->modifier | LockMask, w,
+                                        TRUE, GrabModeAsync, GrabModeSync);
+            status |=
+                XGrabKey (dpy, key->keycode,
+                                        key->modifier | ScrollLockMask | NumLockMask, w,
+                                        TRUE, GrabModeAsync, GrabModeSync);
+            status |=
+                XGrabKey (dpy, key->keycode,
+                                        key->modifier | ScrollLockMask | LockMask, w,
+                                        TRUE, GrabModeAsync, GrabModeSync);
+            status |=
+                XGrabKey (dpy, key->keycode,
+                                        key->modifier | LockMask | NumLockMask, w,
+                                        TRUE, GrabModeAsync, GrabModeSync);
+            status |=
+                XGrabKey (dpy, key->keycode,
+                                        key->modifier | ScrollLockMask | LockMask | NumLockMask, w,
+                                        TRUE, GrabModeAsync, GrabModeSync);
         }
     }
+
+    return (status == GrabSuccess);
 }
 
 void
@@ -181,53 +220,67 @@ ungrabKeys (Display * dpy, Window w)
     XUngrabKey (dpy, AnyKey, AnyModifier, w);
 }
 
-void
+gboolean
 grabButton (Display * dpy, int button, int modifier, Window w)
 {
+    gboolean status;
+
     TRACE ("entering grabButton");
 
+    status = GrabSuccess;
     if (modifier == AnyModifier)
     {
-        XGrabButton (dpy, button, AnyModifier, w, FALSE,
-            ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
-            None, None);
+        status |=
+            XGrabButton (dpy, button, AnyModifier, w, FALSE,
+                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                None, None);
     }
     else
     {
         /* Here we grab all combinations of well known modifiers */
-        XGrabButton (dpy, button, modifier,
-            w, FALSE,
-            ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
-            None, None);
-        XGrabButton (dpy, button, modifier | ScrollLockMask,
-            w, FALSE,
-            ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
-            None, None);
-        XGrabButton (dpy, button, modifier | NumLockMask,
-            w, FALSE,
-            ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
-            None, None);
-        XGrabButton (dpy, button, modifier | LockMask, w, FALSE,
-            ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
-            None, None);
-        XGrabButton (dpy, button, modifier | ScrollLockMask | NumLockMask,
-            w, FALSE,
-            ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
-            None, None);
-        XGrabButton (dpy, button, modifier | ScrollLockMask | LockMask,
-            w, FALSE,
-            ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
-            None, None);
-        XGrabButton (dpy, button, modifier | LockMask | NumLockMask,
-            w, FALSE,
-            ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
-            None, None);
-        XGrabButton (dpy, button,
-            modifier | ScrollLockMask | LockMask | NumLockMask,
-            w, FALSE,
-            ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
-            None, None);
+        status |=
+            XGrabButton (dpy, button, modifier,
+                                w, FALSE,
+                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                None, None);
+        status |=
+            XGrabButton (dpy, button, modifier | ScrollLockMask,
+                                w, FALSE,
+                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                None, None);
+        status |=
+            XGrabButton (dpy, button, modifier | NumLockMask,
+                                w, FALSE,
+                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                None, None);
+        status |=
+            XGrabButton (dpy, button, modifier | LockMask, w, FALSE,
+                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                None, None);
+        status |=
+            XGrabButton (dpy, button, modifier | ScrollLockMask | NumLockMask,
+                                w, FALSE,
+                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                None, None);
+        status |=
+            XGrabButton (dpy, button, modifier | ScrollLockMask | LockMask,
+                                w, FALSE,
+                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                None, None);
+        status |=
+            XGrabButton (dpy, button, modifier | LockMask | NumLockMask,
+                                w, FALSE,
+                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                None, None);
+        status |=
+            XGrabButton (dpy, button,
+                                modifier | ScrollLockMask | LockMask | NumLockMask,
+                                w, FALSE,
+                                ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync,
+                                None, None);
     }
+
+    return (status == GrabSuccess);
 }
 
 void
@@ -281,7 +334,7 @@ initModifiers (Display * dpy)
     keymap = XGetKeyboardMapping (dpy, min_keycode, max_keycode - min_keycode + 1, &keysyms_per_keycode);
 
     if (modmap && keymap)
-    {    
+    {
         for (i = 3 * modmap->max_keypermod; i < 8 * modmap->max_keypermod; i++)
         {
             keycode = modmap->modifiermap[i];
@@ -311,7 +364,7 @@ initModifiers (Display * dpy)
                     else if (!HyperMask && ((syms[j] == XK_Hyper_L) || (syms[j] == XK_Hyper_R)))
                     {
                         HyperMask = (1 << (i / modmap->max_keypermod));
-                    }       
+                    }
                     else if (!MetaMask && ((syms[j] == XK_Meta_L) || (syms[j] == XK_Meta_R)))
                     {
                         MetaMask = (1 << (i / modmap->max_keypermod));
