@@ -1,4 +1,4 @@
-/*      $Id: main.c 28384 2008-10-23 17:18:16Z olivier $
+/*      $Id: main.c 27421 2008-08-04 20:28:44Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,15 +12,17 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/X.h>
@@ -112,6 +114,7 @@ cleanUp (void)
     myDisplayClose (display_info);
     g_free (display_info);
     display_info = NULL;
+    xfconf_shutdown();
 }
 
 static void
@@ -189,7 +192,7 @@ ensure_basedir_spec (void)
         if (w && r)
         {
             int c;
-    
+
             while ((c = getc (r)) != EOF)
             {
                 putc (c, w);
@@ -238,7 +241,7 @@ ensure_basedir_spec (void)
             if (w && r)
             {
                 int c;
-        
+
                 while ((c = getc (r)) != EOF)
                     putc (c, w);
             }
@@ -473,7 +476,7 @@ initialize (int argc, char **argv, gint compositor_mode, gboolean replace_wm)
                 /* Enable compositor if "use compositing" is enabled */
                 xfwm4_compositor = compositorManageScreen (screen_info);
             }
-            /* 
+            /*
                The user may want to use the manual compositing, but the installed
                system may not support it, so we need to double check, to see if
                initialization of the compositor was successful.
