@@ -1,4 +1,4 @@
-/*      $Id: netwm.c 28362 2008-10-22 19:58:39Z olivier $
+/*      $Id: netwm.c 27356 2008-07-20 21:22:15Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,14 +12,16 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/Xlib.h>
@@ -32,6 +34,7 @@
 #include <libxfce4util/libxfce4util.h>
 
 #include "client.h"
+#include "moveresize.h"
 #include "compositor.h"
 #include "display.h"
 #include "frame.h"
@@ -85,7 +88,7 @@ clientSetNetState (Client * c)
         TRACE ("clientSetNetState : skip_taskbar");
         data[i++] = display_info->atoms[NET_WM_STATE_SKIP_TASKBAR];
     }
-    if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED))
     {
         TRACE ("clientSetNetState : maximize vert + horiz");
         data[i++] = display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ];
@@ -331,7 +334,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
             {
                 clientToggleSticky (c, TRUE);
             }
-            frameDraw (c, FALSE);
+            frameQueueDraw (c, FALSE);
         }
     }
 
@@ -411,7 +414,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
             clientSetNetState (c);
             clientWindowType (c);
         }
-        frameDraw (c, TRUE);
+        frameQueueDraw (c, TRUE);
     }
 
     if ((first  == display_info->atoms[NET_WM_STATE_FULLSCREEN]) ||
@@ -445,17 +448,17 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
             if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_ABOVE))
             {
                 FLAG_SET (c->flags, CLIENT_FLAG_ABOVE);
-                clientUpdateAboveState (c);
+                clientUpdateLayerState (c);
             }
             else if ((action == NET_WM_STATE_REMOVE) && FLAG_TEST (c->flags, CLIENT_FLAG_ABOVE))
             {
                 FLAG_UNSET (c->flags, CLIENT_FLAG_ABOVE);
-                clientUpdateAboveState (c);
+                clientUpdateLayerState (c);
             }
             else if (action == NET_WM_STATE_TOGGLE)
             {
                 FLAG_TOGGLE (c->flags, CLIENT_FLAG_ABOVE);
-                clientUpdateAboveState (c);
+                clientUpdateLayerState (c);
             }
         }
     }
@@ -467,21 +470,18 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         {
             if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_BELOW))
             {
-                compositorDamageWindow (display_info, c->frame);
                 FLAG_SET (c->flags, CLIENT_FLAG_BELOW);
-                clientUpdateBelowState (c);
+                clientUpdateLayerState (c);
             }
             else if ((action == NET_WM_STATE_REMOVE) && FLAG_TEST (c->flags, CLIENT_FLAG_BELOW))
             {
-                compositorDamageWindow (display_info, c->frame);
                 FLAG_UNSET (c->flags, CLIENT_FLAG_BELOW);
-                clientUpdateBelowState (c);
+                clientUpdateLayerState (c);
             }
             else if (action == NET_WM_STATE_TOGGLE)
             {
-                compositorDamageWindow (display_info, c->frame);
                 FLAG_TOGGLE (c->flags, CLIENT_FLAG_BELOW);
-                clientUpdateBelowState (c);
+                clientUpdateLayerState (c);
             }
         }
     }
@@ -524,7 +524,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
             FLAG_TOGGLE (c->flags, CLIENT_FLAG_SKIP_TASKBAR);
             clientSetNetState (c);
         }
-        frameDraw (c, TRUE);
+        frameQueueDraw (c, TRUE);
     }
 
     if ((first  == display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION]) ||
@@ -597,8 +597,8 @@ clientNetMoveResize (Client * c, XClientMessageEvent * ev)
         }
         else
         {
-            g_warning ("Could not determine the mouse button used");
-            return;
+            /* Fallback */
+            button = Button1;
         }
     }
 
@@ -608,7 +608,7 @@ clientNetMoveResize (Client * c, XClientMessageEvent * ev)
     event->xbutton.button = button;
     event->xbutton.x_root = event->xkey.x_root = x_root;
     event->xbutton.y_root = event->xkey.y_root = y_root;
-    event->xbutton.time = event->xkey.time = (Time) myDisplayGetCurrentTime (display_info);
+    event->xbutton.time = event->xkey.time = myDisplayGetCurrentTime (display_info);
 
     switch (action)
     {
@@ -686,39 +686,6 @@ clientNetMoveResize (Client * c, XClientMessageEvent * ev)
     }
 }
 
-void
-clientNetMoveResizeWindow (Client * c, XClientMessageEvent * ev)
-{
-    XWindowChanges wc;
-    unsigned long mask;
-    int gravity;
-
-    g_return_if_fail (c != NULL);
-    TRACE ("entering clientNetMoveResizeWindow");
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MOVING_RESIZING))
-    {
-        /* not allowed */
-        return;
-    }
-
-    gravity = (ev->data.l[0] & 0xff);
-    if (!gravity)
-    {
-        gravity = c->gravity;
-    }
-
-    wc.x = ev->data.l[1];
-    wc.y = ev->data.l[2];
-    wc.width = ev->data.l[3];
-    wc.height = ev->data.l[4];
-    mask = (ev->data.l[0] & 0xf00) >> 8;
-
-    clientAdjustCoordGravity (c, gravity, &mask, &wc);
-    clientMoveResizeWindow (c, &wc, mask);
-}
-
 void
 clientUpdateFullscreenState (Client * c)
 {
@@ -756,7 +723,7 @@ clientUpdateFullscreenState (Client * c)
         wc.y = rect.y;
         wc.width = rect.width;
         wc.height = rect.height;
-        layer = WIN_LAYER_ABOVE_DOCK;
+        layer = WIN_LAYER_FULLSCREEN;
     }
     else
     {
@@ -769,15 +736,7 @@ clientUpdateFullscreenState (Client * c)
     clientSetNetState (c);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MANAGED))
     {
-        /*
-           For some reason, the configure can generate EnterNotify events
-           on lower windows, causing a nasty race cond with apps trying to
-           grab focus in focus follow mouse mode. Grab the pointer to
-           avoid these effects
-         */
-        myScreenGrabPointer (c->screen_info, EnterWindowMask, None, myDisplayGetCurrentTime (display_info));
         clientConfigure (c, &wc, CWX | CWY | CWWidth | CWHeight, CFG_FORCE_REDRAW);
-        myScreenUngrabPointer (c->screen_info);
     }
     else
     {
@@ -815,6 +774,7 @@ clientGetNetWmType (Client * c)
     display_info = screen_info->display_info;
     n_atoms = 0;
     atoms = NULL;
+    c->type_atom = None;
 
     if (!getAtomList (display_info, c->window, NET_WM_WINDOW_TYPE, &atoms, &n_atoms))
     {
@@ -830,7 +790,14 @@ clientGetNetWmType (Client * c)
                 c->type_atom = display_info->atoms[NET_WM_WINDOW_TYPE_NORMAL];
                 break;
             default:
-                c->type_atom = None;
+                if (c->transient_for != None)
+                {
+                    c->type_atom = display_info->atoms[NET_WM_WINDOW_TYPE_DIALOG];
+                }
+                else
+                {
+                    c->type_atom = display_info->atoms[NET_WM_WINDOW_TYPE_NORMAL];
+                }
                 break;
         }
     }
@@ -985,7 +952,6 @@ clientValidateNetStrut (Client * c)
     ScreenInfo *screen_info;
     gboolean valid;
     int max_value;
-    int i;
 
     g_return_val_if_fail (c != NULL, TRUE);
     TRACE ("entering clientValidateNetStrut for \"%s\" (0x%lx)", c->name, c->window);
@@ -993,14 +959,31 @@ clientValidateNetStrut (Client * c)
     max_value = MIN (screen_info->width, screen_info->height) / 4;
     valid = TRUE;
 
-    for (i = 0; i < 4; i++)
+    if (c->struts[STRUTS_TOP] > screen_info->height - screen_info->margins[STRUTS_BOTTOM])
     {
-        if (c->struts[i] > max_value)
-        {
-           g_warning ("Strut value for application window 0x%lx changed from %d to %d", c->window, c->struts[i], max_value);
-           c->struts[i] = max_value;
-           valid = FALSE;
-        }
+       c->struts[STRUTS_TOP] = screen_info->height - screen_info->margins[STRUTS_BOTTOM];
+       g_warning ("Top strut value for application window 0x%lx confined to %d", c->window, c->struts[STRUTS_TOP]);
+       valid = FALSE;
+    }
+
+    if (c->struts[STRUTS_BOTTOM] > screen_info->height - screen_info->margins[STRUTS_TOP])
+    {
+       c->struts[STRUTS_BOTTOM] = screen_info->height - screen_info->margins[STRUTS_TOP];
+       g_warning ("Bottom strut value for application window 0x%lx confined to %d", c->window, c->struts[STRUTS_BOTTOM]);
+       valid = FALSE;
+    }
+
+    if (c->struts[STRUTS_LEFT] > screen_info->width - screen_info->margins[STRUTS_RIGHT])
+    {
+       c->struts[STRUTS_LEFT] = screen_info->height - screen_info->margins[STRUTS_RIGHT];
+       g_warning ("Left strut value for application window 0x%lx confined to %d", c->window, c->struts[STRUTS_LEFT]);
+       valid = FALSE;
+    }
+    if (c->struts[STRUTS_RIGHT] > screen_info->width - screen_info->margins[STRUTS_LEFT])
+    {
+       c->struts[STRUTS_RIGHT] = screen_info->height - screen_info->margins[STRUTS_LEFT];
+       g_warning ("Right strut value for application window 0x%lx confined to %d", c->window, c->struts[STRUTS_RIGHT]);
+       valid = FALSE;
     }
 
     return valid;
@@ -1135,7 +1118,7 @@ clientSetNetActions (Client * c)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
-    Atom atoms[6];
+    Atom atoms[16];
     int i;
 
     g_return_if_fail (c != NULL);
@@ -1145,21 +1128,43 @@ clientSetNetActions (Client * c)
     display_info = screen_info->display_info;
     i = 0;
 
+    /* Actions available for all */
     atoms[i++] = display_info->atoms[NET_WM_ACTION_CLOSE];
-    if (CLIENT_CAN_MAXIMIZE_WINDOW (c))
+    atoms[i++] = display_info->atoms[NET_WM_ACTION_ABOVE];
+    atoms[i++] = display_info->atoms[NET_WM_ACTION_BELOW];
+
+    /* Actions depending on the window type and current status */
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
     {
-        atoms[i++] = display_info->atoms[NET_WM_ACTION_MAXIMIZE_HORZ];
-        atoms[i++] = display_info->atoms[NET_WM_ACTION_MAXIMIZE_VERT];
+        atoms[i++] = display_info->atoms[NET_WM_ACTION_FULLSCREEN];
+        if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_MOVE))
+        {
+            atoms[i++] = display_info->atoms[NET_WM_ACTION_MOVE];
+        }
+            if (FLAG_TEST_ALL (c->xfwm_flags, XFWM_FLAG_HAS_RESIZE | XFWM_FLAG_IS_RESIZABLE) &&
+                !FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+        {
+            atoms[i++] = display_info->atoms[NET_WM_ACTION_RESIZE];
+        }
+        if (CLIENT_CAN_MAXIMIZE_WINDOW (c))
+        {
+            atoms[i++] = display_info->atoms[NET_WM_ACTION_MAXIMIZE_HORZ];
+            atoms[i++] = display_info->atoms[NET_WM_ACTION_MAXIMIZE_VERT];
+        }
+        if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER))
+        {
+            atoms[i++] = display_info->atoms[NET_WM_ACTION_SHADE];
+        }
+    }
+    if (CLIENT_CAN_HIDE_WINDOW (c))
+    {
+        atoms[i++] = display_info->atoms[NET_WM_ACTION_MINIMIZE];
     }
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK))
     {
         atoms[i++] = display_info->atoms[NET_WM_ACTION_CHANGE_DESKTOP];
         atoms[i++] = display_info->atoms[NET_WM_ACTION_STICK];
     }
-    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER))
-    {
-        atoms[i++] = display_info->atoms[NET_WM_ACTION_SHADE];
-    }
     XChangeProperty (clientGetXDisplay (c), c->window, display_info->atoms[NET_WM_ALLOWED_ACTIONS],
                      XA_ATOM, 32, PropModeReplace, (unsigned char *) atoms, i);
 }
@@ -1307,36 +1312,19 @@ clientWindowType (Client * c)
 }
 
 void
-clientUpdateAboveState (Client * c)
+clientUpdateLayerState (Client * c)
 {
     int layer;
 
     g_return_if_fail (c != NULL);
-    TRACE ("entering clientUpdateAboveState");
-    TRACE ("Update above state for client \"%s\" (0x%lx)", c->name, c->window);
+    TRACE ("entering clientUpdateLayerState");
+    TRACE ("Update layer state for client \"%s\" (0x%lx)", c->name, c->window);
 
     if (FLAG_TEST (c->flags, CLIENT_FLAG_ABOVE))
     {
         layer = WIN_LAYER_ABOVE_DOCK;
     }
-    else
-    {
-        layer = c->initial_layer;
-    }
-    clientSetNetState (c);
-    clientSetLayer (c, layer);
-}
-
-void
-clientUpdateBelowState (Client * c)
-{
-    int layer;
-
-    g_return_if_fail (c != NULL);
-    TRACE ("entering clientUpdateBelowState");
-    TRACE ("Update below state for client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->flags, CLIENT_FLAG_BELOW))
+    else if (FLAG_TEST (c->flags, CLIENT_FLAG_BELOW))
     {
         layer = WIN_LAYER_BELOW;
     }
@@ -1348,3 +1336,23 @@ clientUpdateBelowState (Client * c)
     clientSetLayer (c, layer);
 }
 
+void
+clientSetNetActiveWindow (ScreenInfo *screen_info, Client *c, Time timestamp)
+{
+    DisplayInfo *display_info;
+    unsigned long data[2];
+
+    g_return_if_fail (screen_info != NULL);
+    TRACE ("entering clientSetNetActiveWindow");
+
+    display_info = screen_info->display_info;
+    data[0] = (unsigned long) None;
+    data[1] = (unsigned long) timestamp;
+    if (c)
+    {
+        data[0] = (unsigned long) c->window;
+    }
+    XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, display_info->atoms[NET_ACTIVE_WINDOW], XA_WINDOW, 32,
+                     PropModeReplace, (unsigned char *) data, 2);
+}
+
