@@ -1,4 +1,4 @@
-/*      $Id: placement.c 26966 2008-05-17 08:22:17Z olivier $
+/*      $Id: placement.c 27548 2008-08-24 16:00:22Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,14 +12,16 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/Xlib.h>
@@ -502,9 +504,13 @@ static void
 clientAutoMaximize (Client * c, int full_w, int full_h)
 {
     if (FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN) ||
-        FLAG_TEST (c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN))
+        FLAG_TEST (c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN) ||
+        !FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER))
     {
-        /* Fullscree nwindows should not be maximized */
+        /*
+         * Fullscreen or undecorated windows should not be
+         * automatically maximized...
+         */
         return;
     }
 
@@ -609,6 +615,22 @@ centerPlacement (Client * c, int full_x, int full_y, int full_w, int full_h)
     c->y = MAX (full_y + frameTop(c) + (full_h - frameHeight(c)) / 2, full_y + frameTop(c));
 }
 
+static void
+mousePlacement (Client * c, int full_x, int full_y, int full_w, int full_h, int mx, int my)
+{
+    g_return_if_fail (c != NULL);
+    TRACE ("entering centerPlacement");
+
+    c->x = mx + frameLeft(c) - frameWidth(c) / 2;
+    c->y = my + frameTop(c) - frameHeight(c) / 2;
+
+    c->x = MIN (c->x, full_x + full_w - frameWidth(c) + frameLeft(c));
+    c->y = MIN (c->y, full_y + full_h - frameHeight(c) + frameTop(c));
+
+    c->x = MAX (c->x, full_x + frameLeft(c));
+    c->y = MAX (c->y, full_y + frameTop(c));
+}
+
 void
 clientInitPosition (Client * c)
 {
@@ -624,8 +646,6 @@ clientInitPosition (Client * c)
     g_return_if_fail (c != NULL);
     TRACE ("entering clientInitPosition");
 
-    clientGravitate (c, APPLY);
-
     screen_info = c->screen_info;
     msx = 0;
     msy = 0;
@@ -633,7 +653,7 @@ clientInitPosition (Client * c)
 
     n_monitors = gdk_screen_get_n_monitors (c->screen_info->gscr);
     monitor_nbr = 0;
-    if (n_monitors > 1)
+    if ((n_monitors > 1) || (screen_info->params->placement_mode == PLACE_MOUSE))
     {
         getMouseXY (screen_info, screen_info->xroot, &msx, &msy);
         monitor_nbr = find_monitor_at_point (screen_info->gscr, msx, msy);
@@ -685,9 +705,19 @@ clientInitPosition (Client * c)
      */
     if (place)
     {
-        if ((screen_info->params->placement_ratio > 100) ||
-            ((frameWidth(c) >= full_w) && (frameHeight(c) >= full_h)) ||
+        if ((screen_info->params->placement_ratio >= 100) ||
             (100 * frameWidth(c) * frameHeight(c)) < (screen_info->params->placement_ratio * full_w * full_h))
+        {
+            if (screen_info->params->placement_mode == PLACE_MOUSE)
+            {
+                mousePlacement (c, full_x, full_y, full_w, full_h, msx, msy);
+            }
+            else
+            {
+                centerPlacement (c, full_x, full_y, full_w, full_h);
+            }
+        }
+        else if ((frameWidth(c) >= full_w) && (frameHeight(c) >= full_h))
         {
             centerPlacement (c, full_x, full_y, full_w, full_h);
         }
