@@ -1,4 +1,4 @@
-/*      $Id: settings.h 25649 2007-04-27 20:01:12Z olivier $
+/*      $Id: settings.h 27421 2008-08-04 20:28:44Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,15 +12,17 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include <glib.h>
@@ -35,68 +37,48 @@
 
 enum
 {
-    TITLE_1,
-    TITLE_2,
-    TITLE_3,
-    TITLE_4,
-    TITLE_5
-};
-
-enum
-{
-    TITLE_SHADOW_NONE,
-    TITLE_SHADOW_UNDER,
-    TITLE_SHADOW_FRAME
-};
-
-enum
-{
-    MENU_BUTTON,
-    STICK_BUTTON,
-    SHADE_BUTTON,
-    HIDE_BUTTON,
-    MAXIMIZE_BUTTON,
-    CLOSE_BUTTON,
-    TITLE_SEPARATOR,
-    BUTTON_COUNT
-};
-#define BUTTON_LAST (BUTTON_COUNT - 1)
-
-enum
-{
-    ACTIVE,
-    INACTIVE,
-    PRESSED,
-    T_ACTIVE,
-    T_INACTIVE,
-    T_PRESSED
+    TITLE_SHADOW_NONE  = 0,
+    TITLE_SHADOW_UNDER = 1,
+    TITLE_SHADOW_FRAME = 2
 };
 
+/** All key definition prior to FIRST_KEY are not grabbed continuously */
+#define FIRST_KEY KEY_ADD_ADJACENT_WORKSPACE
 enum
 {
+    KEY_CANCEL = 0,
+    KEY_DOWN,
+    KEY_LEFT,
+    KEY_RIGHT,
+    KEY_UP,
+    /* FIRST_KEY starts here */
+    KEY_ADD_ADJACENT_WORKSPACE,
     KEY_ADD_WORKSPACE,
-    KEY_CANCEL,
     KEY_CLOSE_WINDOW,
     KEY_CYCLE_WINDOWS,
+    KEY_DEL_ACTIVE_WORKSPACE,
     KEY_DEL_WORKSPACE,
     KEY_DOWN_WORKSPACE,
+    KEY_FILL_HORIZ,
+    KEY_FILL_VERT,
+    KEY_FILL_WINDOW,
     KEY_HIDE_WINDOW,
     KEY_LEFT_WORKSPACE,
     KEY_LOWER_WINDOW,
     KEY_MAXIMIZE_HORIZ,
     KEY_MAXIMIZE_VERT,
     KEY_MAXIMIZE_WINDOW,
-    KEY_MOVE_DOWN,
+    KEY_MOVE,
     KEY_MOVE_DOWN_WORKSPACE,
-    KEY_MOVE_LEFT,
     KEY_MOVE_LEFT_WORKSPACE,
     KEY_MOVE_NEXT_WORKSPACE,
     KEY_MOVE_PREV_WORKSPACE,
-    KEY_MOVE_RIGHT,
     KEY_MOVE_RIGHT_WORKSPACE,
-    KEY_MOVE_UP,
     KEY_MOVE_UP_WORKSPACE,
     KEY_MOVE_WORKSPACE_1,
+    KEY_MOVE_WORKSPACE_10,
+    KEY_MOVE_WORKSPACE_11,
+    KEY_MOVE_WORKSPACE_12,
     KEY_MOVE_WORKSPACE_2,
     KEY_MOVE_WORKSPACE_3,
     KEY_MOVE_WORKSPACE_4,
@@ -105,21 +87,16 @@ enum
     KEY_MOVE_WORKSPACE_7,
     KEY_MOVE_WORKSPACE_8,
     KEY_MOVE_WORKSPACE_9,
-    KEY_MOVE_WORKSPACE_10,
-    KEY_MOVE_WORKSPACE_11,
-    KEY_MOVE_WORKSPACE_12,
     KEY_NEXT_WORKSPACE,
     KEY_POPUP_MENU,
     KEY_PREV_WORKSPACE,
     KEY_RAISE_WINDOW,
-    KEY_RESIZE_DOWN,
-    KEY_RESIZE_LEFT,
-    KEY_RESIZE_RIGHT,
-    KEY_RESIZE_UP,
+    KEY_RESIZE,
     KEY_RIGHT_WORKSPACE,
     KEY_SHADE_WINDOW,
     KEY_SHOW_DESKTOP,
     KEY_STICK_WINDOW,
+    KEY_TOGGLE_ABOVE,
     KEY_TOGGLE_FULLSCREEN,
     KEY_UP_WORKSPACE,
     KEY_WORKSPACE_1,
@@ -156,9 +133,16 @@ enum
     DBL_CLICK_ACTION_NONE,
     DBL_CLICK_ACTION_MAXIMIZE,
     DBL_CLICK_ACTION_SHADE,
+    DBL_CLICK_ACTION_FILL,
     DBL_CLICK_ACTION_HIDE
 };
 
+enum
+{
+    PLACE_MOUSE,
+    PLACE_CENTER
+};
+
 struct _XfwmColor
 {
     GdkColor col;
@@ -168,15 +152,16 @@ struct _XfwmColor
 
 struct _Settings
 {
-    gchar *option;
-    gchar *value;
+    gchar  *option;
+    GValue *value;
+    GType   type;
     gboolean required;
 };
 
 struct _XfwmParams
 {
     MyKey keys[KEY_COUNT];
-    gchar button_layout[BUTTON_COUNT + 1];
+    gchar button_layout[BUTTON_STRING_COUNT + 1];
     unsigned int xfwm_margins[4];
     int activate_action;
     int button_offset;
@@ -188,8 +173,10 @@ struct _XfwmParams
     int maximized_offset;
     int move_opacity;
     int placement_ratio;
+    int placement_mode;
     int popup_opacity;
     int raise_delay;
+    int focus_delay;
     int resize_opacity;
     int restore_on_move;
     int shadow_delta_height;
@@ -217,6 +204,7 @@ struct _XfwmParams
     gboolean raise_with_any_button;
     gboolean scroll_workspaces;
     gboolean show_app_icon;
+    gboolean show_dock_shadow;
     gboolean show_frame_shadow;
     gboolean show_popup_shadow;
     gboolean snap_to_border;
