@@ -1,4 +1,4 @@
-/*      $Id: wireframe.h 23704 2006-11-04 22:36:42Z olivier $
+/*      $Id: wireframe.h 26999 2008-05-31 14:35:42Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,28 +12,30 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         metacity - (c) 2001 Anders Carlsson, Havoc Pennington
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
+#ifndef INC_WIREFRAME_H
+#define INC_WIREFRAME_H
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#ifndef INC_WIREFRAME_H
-#define INC_WIREFRAME_H
-
 #include <X11/Xlib.h>
 #include "screen.h"
 #include "client.h"
 
-void                     wireframeUpdate                        (Client *c, 
+void                     wireframeUpdate                        (Client *c,
                                                                  Window);
 Window                   wireframeCreate                        (Client *c);
-void                     wireframeDelete                        (ScreenInfo *, 
+void                     wireframeDelete                        (ScreenInfo *,
                                                                  Window);
 
 #endif /* INC_WIREFRAME_H */
