@@ -1,4 +1,4 @@
-/*      $Id: workspaces.c 28362 2008-10-22 19:58:39Z olivier $
+/*      $Id: workspaces.c 27077 2008-06-16 21:17:09Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,15 +12,17 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/X.h>
@@ -176,10 +178,14 @@ modify_with_wrap (int value, int by, int limit, gboolean wrap)
 
 /* returns TRUE if the workspace was changed, FALSE otherwise */
 gboolean
-workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c, guint32 timestamp)
+workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c, Time timestamp)
 {
     int row, col, newrow, newcol, previous_ws, n;
 
+    g_return_val_if_fail (screen_info != NULL, FALSE);
+
+    TRACE ("entering workspaceMove");
+
     workspaceGetPosition (screen_info, screen_info->current_ws, &row, &col);
     newrow = modify_with_wrap (row, rowmod, screen_info->desktop_layout.rows, screen_info->params->wrap_layout);
     newcol = modify_with_wrap (col, colmod, screen_info->desktop_layout.cols, screen_info->params->wrap_layout);
@@ -229,7 +235,7 @@ workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c, guin
 }
 
 void
-workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean update_focus, guint32 timestamp)
+workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean update_focus, Time timestamp)
 {
     DisplayInfo *display_info;
     Client *c, *new_focus;
@@ -241,6 +247,8 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean upda
     unsigned int mask;
     unsigned long data[1];
 
+    g_return_if_fail (screen_info != NULL);
+
     TRACE ("entering workspaceSwitch");
 
     display_info = screen_info->display_info;
@@ -270,8 +278,6 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean upda
         return;
     }
 
-    myScreenGrabPointer (screen_info, EnterWindowMask, None, timestamp);
-
     screen_info->previous_ws = screen_info->current_ws;
     screen_info->current_ws = new_ws;
 
@@ -321,7 +327,7 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean upda
     for (index = screen_info->windows_stack; index; index = g_list_next (index))
     {
         c = (Client *) index->data;
-        
+
         if (c->win_workspace != new_ws)
         {
             if (c == previous)
@@ -343,7 +349,7 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean upda
     for (index = g_list_last(screen_info->windows_stack); index; index = g_list_previous (index))
     {
         c = (Client *) index->data;
-        
+
         if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
         {
             if ((!new_focus) && (c == previous) && clientSelectMask (c, 0, WINDOW_REGULAR_FOCUSABLE))
@@ -397,13 +403,16 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean upda
             clientFocusTop (screen_info, WIN_LAYER_NORMAL, timestamp);
         }
     }
-
-    myScreenUngrabPointer (screen_info);
 }
 
 void
 workspaceSetNames (ScreenInfo * screen_info, gchar **names, int items)
 {
+    g_return_if_fail (screen_info != NULL);
+    g_return_if_fail (names != NULL);
+
+    TRACE ("entering workspaceSetNames");
+
     if (screen_info->workspace_names)
     {
         g_strfreev (screen_info->workspace_names);
@@ -420,6 +429,8 @@ workspaceSetCount (ScreenInfo * screen_info, int count)
     Client *c;
     int i;
 
+    g_return_if_fail (screen_info != NULL);
+
     TRACE ("entering workspaceSetCount");
 
     if (count < 1)
@@ -454,6 +465,60 @@ workspaceSetCount (ScreenInfo * screen_info, int count)
                      &screen_info->desktop_layout);
 }
 
+void
+workspaceInsert (ScreenInfo * screen_info, int index)
+{
+    Client *c;
+    int i, count;
+
+    g_return_if_fail (screen_info != NULL);
+
+    TRACE ("entering workspaceInsert");
+
+    count = screen_info->workspace_count;
+    workspaceSetCount(screen_info, count + 1);
+
+    if ((index < 0) || (index > count))
+    {
+        return;
+    }
+
+    for (c = screen_info->clients, i = 0; i < screen_info->client_count; c = c->next, i++)
+    {
+        if (c->win_workspace >= index)
+        {
+            clientSetWorkspace (c, c->win_workspace + 1, TRUE);
+        }
+    }
+}
+
+void
+workspaceDelete (ScreenInfo * screen_info, int index)
+{
+    Client *c;
+    int i, count;
+
+    g_return_if_fail (screen_info != NULL);
+
+    TRACE ("entering workspaceDelete");
+
+    count = screen_info->workspace_count;
+    if ((index < 0) || (index > count))
+    {
+        return;
+    }
+
+    for (c = screen_info->clients, i = 0; i < screen_info->client_count; c = c->next, i++)
+    {
+        if (c->win_workspace > index)
+        {
+            clientSetWorkspace (c, c->win_workspace - 1, TRUE);
+        }
+    }
+
+    workspaceSetCount(screen_info, count - 1);
+}
+
 void
 workspaceUpdateArea (ScreenInfo *screen_info)
 {
